/***************************************************************************
*									   *
*	File : complex.c						   *
*	Nore : Basic complex operations.				   *
*									   *
***************************************************************************/

#include	<math.h>
#include	"complex.h"

cmplx complex ( double x, double y )		/* (x, jy)  */
{
	cmplx	a;
	a.re = x;
	a.im = y;
	return ( a );
}

cmplx cmplx_add ( cmplx a, cmplx b )		/* a + b */
{
	cmplx c;
	c.re = a.re+b.re;
	c.im = a.im+b.im;
	return( c );
}

cmplx cmplx_sub ( cmplx a, cmplx b )		/*  a - b  */
{
	cmplx c;
	c.re = a.re-b.re;
	c.im = a.im-b.im;
	return( c );
}

cmplx cmplx_scl ( double x, cmplx a )		/*  x*a  */
{
	cmplx c;
	c.re = x*a.re;
	c.im = x*a.im;
	return( c );
}

cmplx cmplx_mul ( cmplx a, cmplx b )		/*  a*b  */
{
	cmplx c;
	c.re = a.re*b.re-a.im*b.im;
	c.im = a.re*b.im+a.im*b.re;
	return( c );
}

cmplx cmplx_div ( cmplx a, cmplx b )		/*  a/b  */
{
	double d;
	cmplx c;
	d = b.re*b.re + b.im*b.im;
	c.re = (a.re*b.re+a.im*b.im)/d;
	c.im = (a.im*b.re-a.re*b.im)/d;
	return( c );
}

cmplx cmplx_add_mul ( cmplx a, cmplx b, cmplx c )	/* a + b*c */
{
	cmplx d;
	d.re = a.re+b.re*c.re-b.im*c.im;
	d.im = a.im+b.re*c.im+b.im*c.re;
	return( d );
}

cmplx cmplx_sub_mul ( cmplx a, cmplx b, cmplx c )	/* a - b*c */
{
	cmplx d;
	d.re = a.re-b.re*c.re+b.im*c.im;
	d.im = a.im-b.re*c.im-b.im*c.re;
	return( d );
}

cmplx cmplx_inv ( cmplx a )			/* 1/a */
{
	double d;
	cmplx c;
	d = a.re*a.re + a.im*a.im;
	c.re = a.re/d;
	c.im = -a.im/d;
	return( c );
}

double abs_cmplx ( cmplx a )			/* |a| */
{
	return( sqrt(a.re*a.re + a.im*a.im) );
}

double pha_cmplx ( cmplx a )			/* phase angle */
{
	return( atan2(a.im, a.re) );
}
