/**************************************************************************
 * General info:  This code was rewritten by Frank G in 8/96 to allow for
 * operation under linux.  The smallest sound DMA buffer currently available
 * for linux at 8kHz, 16-bit samples is 2048 bytes, which is 128 ms
 * of sound.  Linux will not play sound until the entire DMA buffer 
 * is full; i.e., we're limited to 128ms chunks of sound, and there is an
 * apparently unavoidable 128ms delay between the video and the audio
 * under linux.  The  utterance length and frame length for sensyn have 
 * both been set to 4 ms (set in * diva_sensyn.c, which overrides the 
 * values in the default60.con file).  Other values should work as long 
 * as 128/utterance length is an integer, but this might require some 
 * small fixes to the code.
***************************************************************************/

#include	"always.h"
#include	"../main_src/diva.h"
#include	"complex.h"
#include	"plot_lib.h"
#include	"lam_lib.h"
#include	"vtconfig.h"
#include	"vsyn_lib.h"
#include	"maeda_dir.h"
/*
#include "../rbf_src/rbf.h"
*/

#include	<fcntl.h>
#include	<sys/ioctl.h>
#include	<sys/stat.h>
#include	<sys/types.h>
#include	<sys/time.h>

#ifdef LINUX_FLAG
 #include <linux/soundcard.h>
 #include <fcntl.h>
#else
 #include <sun/audioio.h>
#endif

/***********************( prototype definitions )**************************/

void	read_rad( void );
void    store_maeda(char* full_sound_buffer, int buffer_size);
void    playback_maeda( void );
void    playback_maeda_articulators( void );
void    draw_maeda( void );
void	write_sound_buf(char* bufptr, int sampleCount);
void    find_area_function( void );
void	draw_markers( void );
void	open_audio_device( void );
int	select (int, fd_set*, fd_set*, fd_set*, struct timeval *);
void	update_maeda_sliders( void );
void	display_formants( void );
int     gettimeofday();

/**********( global variables seen by functions in "maeda.c" )*********/

	int	maeda_init_flag=0;
	int	maeda_count=0;
	int	af_calloc_flag=0;
	FILE*	sound_file_ptr = NULL;
	FILE*	dft_fp = NULL;
	double	syn_targets[NUM_TARGETS]={220.0, 1220.0, 2220.0/*, 600.0*/};
	extern	int sound_buf_count;
	extern	int erase_flag;
	int	synthesizing=0;
	int	record_demo=0;
	int	audio_file_open_flag=0;
	int	audio_fd;
	double	AMpar[7] ={0., 0., 0., 0., 0., 0., 0.};
	char sound_storage_buffer[2048];  /* buffer for storing sound
					     samples until enough frames
					     have been created. */
	int current_buffer_size = 0;  /* size of buffer for storing
					 sound samples -- used by 
					 process_sound_frame() and 
					 flush_sound() */
        int bytes_in_DMA_buffer = 0; /* Keeps track of how many bytes
					  are in DMA buffer; updated by
					  write_sound_buf and used by
					  flush_sound() */

/*********************( global plotting variables )************************/

	int	nfrms, nfrmmax = 4;	/* number of formant frequencies  */
	double	frm[8], bw[8], amp[8];	/* formant freqs. and amplitude   */

	vp_lo_frame vp1 = { 1, 0.5, 5.0, 49., 25. }; 	/* area_function  */
	vp_lo_frame vp2 = { 2, 0.5, 32., 49., 37. };    /* VT transfer	  */
	vp_lo_frame vp3 = { 3, 50., 42., 29., 23. };	/* formants	  */
	vp_lo_frame vp4 = { 4, 55., 5.,  35., 35. };	/* VT profile	  */

	char	*UmUg_title = "(Um + Un)/Ug TRANSFER RATIO";
	char	*UmPt_title = "(Um + Un)/Pt TRANSFER RATIO";

/* normalized radiation impedance */
        static  float   *rad_re, *rad_im;
        static  float    dka;
        static  int      n_ka;
        static  double   lip_Rrad, lip_Lrad;
        static  double   nose_Rrad, nose_Lrad;
        static  double   lip_rad_ka, lip_rad_norm;
        static  double   nose_rad_ka, nose_rad_norm;




/******
*	Function : process_sound_frame() -- formerly named maeda()
*    
*       Processes each sensyn sound frame (i.e., set of sound samples
*       between changes of the sensyn parameters, currently equal to 4
*       sound samples) after the parameters have been modified by in
*       speak() or babble().  Checks to see if enough samples have 
*       been stored, and if so, stores them if necessary and sends 
*       them to the audio device (currently one DMA buffer full is
*       32 sensyn frames = 128 ms of sound at 8kHz = 1024 16-bit sound 
*       samples = 2048  characters in buffer).	find_area_function() 
*       must be called before calling process_sound_frame().  This is 
*       usually done in apply_methods().
******/
void    process_sound_frame( void )
{
	XEvent	event;
	int	i;
	static int next_start_time = 0;  /* needed for real-time operation */
	int	formants[6];
	struct	timeval tv;
	struct timezone tz;
	extern char sound_buf[];  /* not currently used, but needed for
				     8-bit ulaw encoding (set by writwave.c).
				     This was replaced by 16-bit linear
				     encoding by Frank G, 8/96 */

	extern short sound_buf16[]; /* replaces sound_buf in newer version */
	
	while (XtPending() != 0)
	  {
	    XtNextEvent(&event);
	    XtDispatchEvent(&event);
	  }

      	if (sound_on == 1.0)
	  {
	    /* compute speech samples */
	    for(i=0; i<NUM_TARGETS; i++)
	      {
		formants[i] = (int)syn_targets[i];
	      }

	    sensyn( formants );  /* puts one frame of sound in sound_buf 
				    and sound_buf16 */

	    /* OLD VERSION -- 8-bit ULAW */
	    /* memcpy((void *)(sound_storage_buffer + current_buffer_size), 
		   sound_buf, sound_buf_count);
               current_buffer_size = current_buffer_size + sound_buf_count; */

	    /* NEW VERSION -- 16-bit linear encoding */
	    memcpy((void *)(sound_storage_buffer + current_buffer_size), 
		   sound_buf16, (2 * sound_buf_count));
            current_buffer_size = current_buffer_size + (2 * sound_buf_count);

	    /* 2048 is DMA buffer size for linux at 8kHz 16-bit samples.  
	       To promote smoothness in the animation, however, will send
	       the graphics and sound when buffer size is a multiple of 512 
	       (i.e., every 32 ms).  flush_sound() should be called at the 
	       end of the utterance to make sure the rest of the sound 
	       buffer has been removed. */

	    if (current_buffer_size  == 1024)
	      {
		current_buffer_size = 0;
		if (record_demo == 1)
		  store_maeda(sound_storage_buffer, 1024); 
		/* Wait til it's time to send sound buffer to audio device.
		   The second condition on the following is necessary
		   to avoid trouble when next_start_time wraps around */
		do
		  {
		    gettimeofday(&tv,&tz);
		  } while ((tv.tv_usec < next_start_time) || 
		       (tv.tv_usec > next_start_time + 500000));
		
		/* If first sample in an utterance (or if delayed for some
		   reason), start next sample right away. If not, send 
		   next sample in 64 ms. */
		if (tv.tv_usec > (next_start_time + 64000)) /* 1st in series */
		  next_start_time = tv.tv_usec;
		else
		  next_start_time = next_start_time + 64000; 
		if (next_start_time >= 1000000)
		  next_start_time = next_start_time - 1000000;
		
		/* Now send sound buffer to audio device */
		write_sound_buf(sound_storage_buffer, 1024);

		/* Draw the corresponding articulator position */
		playback_maeda_articulators();
	      }
	  }
	else 
	  {
	    draw_maeda();
	    draw_markers();
	    XFlush(theDisplay);
	    return;
	  }
}



/********
*    Function:  flush_sound() 
*
*    Called in place of process_sound_frame() at the end of an utterance 
*    that might not have totally filled up the sound DMA buffer (2048 bytes).
*
*********/
void flush_sound(void)
{
      int i;
  
      if (sound_on == 0) return;

      while (bytes_in_DMA_buffer != 0) /* set by write_sound_buf */
	{
	  sound_storage_buffer[current_buffer_size] = 0;	
	  current_buffer_size = current_buffer_size + 1;
	  if (current_buffer_size == 1024)
	    {
	      playback_maeda_articulators();
	      if (record_demo == 1)
		store_maeda(sound_storage_buffer, 1024);
	      write_sound_buf(sound_storage_buffer, 1024);
	      current_buffer_size = 0;
	    }
	}
}


/******
*	Function : playback_maeda_articulators()
*
*	This is just a faster version of draw_maeda_articulators(), but
*	which doesn't compute and plot formants, doesn't plot the
*	area function, and doesn't redraw the markers, all of which load
*	the system and cause annoying stops in the acoustic output.
******/
void    playback_maeda_articulators( void )
{
	find_area_function();
	deplot_lam();
	plot_lam();

	XFlush(theDisplay);
}



/******
*	Function : draw_maeda_articulators()
******/
void    draw_maeda_articulators( void )
{

  /*
int i;
        for(i=0;i<7;i++)
            printf("%lf ",AMpar[i]);
        printf("\n");
        if(ok)
         PrintBias(P,AMpar);
  */
	find_area_function();
	get_formants();
	draw_maeda();
	draw_markers();
	XFlush(theDisplay);
}



/*********************************************************************
*	Function : store_maeda()
*
*	This function currently stores one chunk of sound (currently
*       32 ms, or 512 bytes), along with corresponding articulator position 
*       and formant information, in a file for playback.  
*       This function is called only by process_sound_frame() or flush_sound(),
*       which must first insure that enough samples have been stored up.
**********************************************************************/
void    store_maeda( char* sound_buffer, int buffer_size )
{
	int	i;
	float	temp;

	/* store the 3 formants in the file */
	fwrite(syn_targets, sizeof(syn_targets[0]),
	       (int)3, sound_file_ptr);
	
	/* store the 7 maeda parameters in the file */
	fwrite(AMpar, sizeof(AMpar[0]), (int)7, sound_file_ptr);

	/* write next sound buffer to file */
	fwrite(&buffer_size, sizeof(int), (int)1, sound_file_ptr);
	fwrite(sound_buffer, sizeof(char),
	       buffer_size, sound_file_ptr);
}


/******************************************************************/
/**	Function : play_demo() --  Read from a stored demo       **/
/**     file the maeda parameters and sound samples for each     **/
/**	frame, draw the articulators, and play the sound.        **/
/**     8/96 -- modified to work under linux and to use 16-bit   **/
/**     linear encoding.  To switch back to 8-bit encoding       **/
/**     (linear or ULAW), need to double the 32ms and 64ms delay **/
/**     times in the following.                                  **/
/******************************************************************/
void    play_demo( FILE *sound_file_ptr )
{
	int	loop_count = 0;
	int     next_start_time = 0;
	int	num_read;
	char	sound_buf[8000];
	int     sampleCount;
	int     first_time = 1;
	int	err;
	int	skipcount=9;
	struct	timeval tv;
	struct timezone tz;

	XtSetSensitive(play_demo_button, FALSE);
	XtSetSensitive(make_demo_button, FALSE);
	XtSetSensitive(speak_button, FALSE);
	XtSetSensitive(babble_button, FALSE);
	XtSetSensitive(formant_button, FALSE);
	XtSetSensitive(ratio_button, FALSE);
	XtSetSensitive(pause_button, FALSE);
	XtSetSensitive(synthesize_button, FALSE);
	XtSetSensitive(cap_button, FALSE);
	XtSetSensitive(fix_jaw_button, FALSE);
	XtSetSensitive(load_weights_button, FALSE);
	XtSetSensitive(load_parameters_button, FALSE);
	XtSetSensitive(plot_conf_button, FALSE);

	open_audio_device();

#ifdef COMMENT	
	/* In linux, flush out audio device */
	#ifdef LINUX_FLAG
	if (ioctl(audio_fd, SOUND_PCM_RESET, 0) == -1) 
	  printf("I/O error:  ioctl to audio device failed.\n");
        #endif
	bytes_in_DMA_buffer = 0;
#endif COMMENT

	/*
	 * Read from the file:
	 *
	 * Read maeda parameters and sound samples for each
	 * frame, draw the articulators, and play the sound.
	 */
	while (1) 
	  {
	    num_read=fread(syn_targets, sizeof(syn_targets[0]), (int)3,
	    sound_file_ptr);
	    if (num_read == 0) break;
	    num_read=fread(AMpar,sizeof(AMpar[0]),(int)7,sound_file_ptr);
	    if (num_read == 0) break;

	    /* Read next sound buffer from file */
	    num_read=fread(&sampleCount,sizeof(int),(int)1,sound_file_ptr);
	    if (num_read == 0) break;
	    num_read=fread(sound_buf,sizeof(char),sampleCount,
			   sound_file_ptr);
	    if (num_read == 0) break;

	    /* Wait til it's time to send sound buffer to audio device */
	    if (first_time)
	      {
		first_time = 0;
		gettimeofday(&tv,&tz);
		/* send first sample now, second sample in 32 ms */
		next_start_time = tv.tv_usec + 32000;
		if (next_start_time >= 1000000)
		  next_start_time = next_start_time - 1000000;
	      }
	    else
	      {
		/* The second condition on the following is necessary
		   to avoid trouble when next_start_time wraps around */
		while ((tv.tv_usec < next_start_time) || 
		       (tv.tv_usec > next_start_time + 500000))
		  {
		    gettimeofday(&tv,&tz);
		  }
		/* send next sample in 64 ms */
                if (tv.tv_usec < next_start_time + 64000)
		  next_start_time = next_start_time + 64000; 
		else /* catch up now if delayed for some reason */
		  next_start_time = tv.tv_usec;
		if (next_start_time >= 1000000)
		  next_start_time = next_start_time - 1000000;
	      }

	    /* Now send sound buffer to audio device */
	    if (sound_on > 0.1)
	      write_sound_buf(sound_buf, sampleCount);

	    /* Draw the corresponding articulator position */
	    playback_maeda_articulators();
	  }
	
	fclose(sound_file_ptr);
	sound_file_ptr = NULL;
	
	XtSetSensitive(speak_button, TRUE);
	XtSetSensitive(babble_button, TRUE);
	XtSetSensitive(formant_button, TRUE);
	XtSetSensitive(ratio_button, TRUE);
	XtSetSensitive(synthesize_button, TRUE);
	XtSetSensitive(pause_button, TRUE);
	XtSetSensitive(cap_button, TRUE);
	XtSetSensitive(fix_jaw_button, TRUE);
	XtSetSensitive(load_weights_button, TRUE);
	XtSetSensitive(load_parameters_button, TRUE);
	XtSetSensitive(plot_conf_button, TRUE);
	XtSetSensitive(play_demo_button, TRUE);
	XtSetSensitive(make_demo_button, TRUE);

	shift_eye();
	flap_arms();

	update_maeda_sliders();
	display_formants();
}



/*********************************************************************/
/**	Function : find_area_function() -- set up some stuff, then  **/
/**     call lam() to get vt profile and sagittal_to_area() and     **/
/**	appro_area_function() to get area function (these functions **/
/**     are in lam_lib.c).                                          **/
/*********************************************************************/
void    find_area_function( void )
{
		int	i,j, p=0;
		int	ns0 = 29;
	static	area_function	*af0;

	if (af_calloc_flag==0) {
		af0 = (area_function *) calloc( ns0, sizeof(area_function) );
		nph = 9;
		nbu = 8;
		nss = nbu + nph;
		afvt  = (area_function *) calloc( nss, sizeof(area_function) );
		af_calloc_flag = 1;
	}

	/* Compute VT profile and area function */
	lam( AMpar );				/* profile	*/
	sagittal_to_area( &ns0, af0 );		/* area function */

	appro_area_function( ns0, af0, nss, afvt);

	if( nasal_tract == ON )
	{  anc = (double) min( anc, afvt[nph].A );
	   afvt[nph].A -= anc;
	}
}


/*********************************************************************/
/**	Function : draw_maeda() -- erases (usually) and draws maeda **/
/**     articulators and area function (using plot_lam() and        **/
/**     plot_af(),respectively, from lam_lib.c).                    **/
/**     find_area_function() must be called before draw_maeda().    **/
/*********************************************************************/
void    draw_maeda( void )
{
	/* draw maeda articulators */
	if (erase_flag) deplot_lam();
	plot_lam();

	/* plot area function */
	deplot_af();
	plot_af ( nss, afvt );
}



void init_maeda( void )
{
	static  char    *nt_filename = "nt.are";
	static  char	af_path[40];
	int	j;

	maeda_count = 0;

	if (maeda_init_flag == 0) {
		/* read nasal tract area function from the file */
		strcpy(af_path, VTAFdir);
		strcat(af_path, nt_filename);
		read_af( af_path, &nna, &afnt );
		read_rad();

		/* Initialization */
	 	read_short_model_spec();
		/* read_model_spec();  Replaced by FG; 7/10/96 */
		convert_scale();
		semi_polar();
		/* init_rbf(); */
		for (j=0; j<7; j++) AMpar[j] = 0.0;

		maeda_init_flag = 1;
	}
}



void write_sound_buf(char* bufptr, int sampleCount)
{
    int err;

    if (sound_on == 0.0) return;
    open_audio_device();
    if (write(audio_fd, bufptr, sampleCount) == -1)
      printf("Write to audio device failed.\n");
    bytes_in_DMA_buffer = bytes_in_DMA_buffer + sampleCount;
    if (bytes_in_DMA_buffer == 2048)
      bytes_in_DMA_buffer = 0;
}



void	open_audio_device( void )
{
#ifdef LINUX_FLAG

  int div = 4, sample_bits=16;

  if (audio_file_open_flag == 0) {
    if ((audio_fd=open("/dev/dsp", O_RDWR)) == -1) {
      printf("can't open /dev/dsp\n");
      exit(1);
    }
    audio_file_open_flag = 1;
    /* Shorten DMA buffer to 2048 bytes (at 8kHz, 16-bit samples) for linux */
    if (ioctl(audio_fd, SOUND_PCM_SUBDIVIDE, &div) == -1) 
      printf("I/O error:  ioctl to audio device failed.\n");
    /* Set up for 16 bit samples (linear encoding is default for /dev/dsp) */
    if (ioctl(audio_fd, SOUND_PCM_WRITE_BITS, &sample_bits) == -1) 
      printf("I/O error:  ioctl to audio device failed.\n");
    /* Flush any old stuff out of the DMA buffer -- probably unnecessary */
    if (ioctl(audio_fd, SOUND_PCM_RESET, 0) == -1) 
      printf("I/O error:  ioctl to audio device failed.\n");
    bytes_in_DMA_buffer = 0;
    /* Print DMA buffer size 
    ioctl(audio_fd, SNDCTL_DSP_GETBLKSIZE, &div);
    printf("DMA buffer size:  %d\n",div);
    */
  }

#else 
  
  audio_info_t info;

  if (audio_file_open_flag == 0) {
    if ((audio_fd=open("/dev/audio", O_RDWR)) == -1) {
      printf("can't open /dev/audio\n");
      exit(1);
    }
    audio_file_open_flag = 1;
    /* Set up for 8kHz, 16 bit samples and linear encoding */
    AUDIO_INITINFO(&info);
    info.play.sample_rate = 8000;
    info.play.channels = 1;
    info.play.encoding = AUDIO_ENCODING_LINEAR;
    info.play.precision = 16;
    if (ioctl(audio_fd, AUDIO_SETINFO, &info) == -1) 
      printf("I/O error:  ioctl to audio device failed.\n");

  }
  
#endif
}
