#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include "fast_maeda.h"
#include "../main_src/diva_dir.h"

int md_malloc_flag;
short *md_ptr;

/*
 * This function implements a generalization of bilinear interpolation as
 * described in "Numerical Recipes in C (2nd Ed)", page 123.
 *
 * This function returns 1 if successful, 0 otherwise.
 */
int interpolate_dict( double inpar[7], double *out_frm )
{
	int	i,j,k,l,m,n,p,q;
	int	r[DIM];
	double	f[DIM],d[DIM];
	double	prod;
	double	par[7];
	short*	md;

	if (md_malloc_flag == 0)
		return(0);

	/* to make the binning work just right, don't let inpar[]=MAX_ARTIC */
	for (i=0; i<7; i++) {
		par[i] = inpar[i];
		if (par[i] >= MAX_ARTIC)
			par[i] = MAX_ARTIC-0.0001;
		else
			if (par[i] < -MAX_ARTIC)
				par[i] = -MAX_ARTIC;
	}

	for (i=0; i<DIM; i++) {
		r[i] = (int)((par[i]+MAX_ARTIC)/ARTIC_STEP);
		/* Equation 3.6.4, page 123 */
		f[i] = (par[i] - (-MAX_ARTIC + r[i]*ARTIC_STEP))/ARTIC_STEP;
		/* corresponds to (1-t) or (1-u), page 123 */
		d[i] = 1 - f[i];
	}

	for (i=0; i<NUM_FORMANTS; i++)
		out_frm[i] = 0;

	for (i=0; i<2; i++)
	 for (j=0; j<2; j++)
	  for (k=0; k<2; k++)
	   for (l=0; l<2; l++)
	    for (m=0; m<2; m++)
	     for (n=0; n<2; n++)
	      for (p=0; p<2; p++) {

		prod = (i ? f[0]: d[0]) * (j ? f[1]: d[1]) *
		           (k ? f[2]: d[2]) * (l ? f[3]: d[3]) *
		           (m ? f[4]: d[4]) * (n ? f[5]: d[5]) *
			   (p ? f[6]: d[6]);

		md = md_ptr;
		md += 	NUM_FORMANTS*(r[6]+p) +
			+ NP * NUM_FORMANTS*(r[5]+n) +
			+ NP * NP * NUM_FORMANTS*(r[4]+m) +
			+ NP * NP * NP * NUM_FORMANTS*(r[3]+l) +
			+ NP * NP * NP * NP * NUM_FORMANTS*(r[2]+k) +
			+ NP * NP * NP * NP * NP * NUM_FORMANTS*(r[1]+j) +
			+ NP * NP * NP * NP * NP * NP * NUM_FORMANTS*(r[0]+i);

		for (q=0; q<NUM_FORMANTS; q++) {
			if ((*md) == 8000 && prod != 0.0)
				return(0);
			out_frm[q] += (*md) * prod;
			md++;
		}
	      }
	return(1);
}


load_dict()
{
	FILE*	fp;
	int	num_elements,i;
/* Next two lines will cause us to use the Maeda calculations */
/*
	md_malloc_flag=0;
	return;
*/
	if ((md_ptr=(short*)malloc( sizeof(short)*ARRAY_SIZE )) == NULL) {
		printf("can't malloc space for maeda dictionary\n");
		md_malloc_flag=0;
		return;
	}
	md_malloc_flag=1;

	if((fp = fopen(MD7,"r")) == NULL) {
		printf("can't open '%s'\n", MD7);
		free(md_ptr);
		md_malloc_flag=0;
		return;
	}

	num_elements = fread(md_ptr, sizeof(short), ARRAY_SIZE, fp) ;
	fclose(fp) ;

	if (num_elements != ARRAY_SIZE) {
		printf("invalid md7.dat file\n");
		free(md_ptr);
		md_malloc_flag=0;
		return;
	}

	/*  Have to switch the byte ordering in linux since SUN OS
	    uses big endian and linux uses little endian or vice versa */
	#ifdef LINUX_FLAG
	swab(md_ptr,md_ptr,(2*ARRAY_SIZE));
        #endif LINUX_FLAG
}
