/***************************************************************************
*									   *
*	File :	plot_lib.c						   *
*	Note :	plotting functions for vocal tract calculation programs	   *
*									   *
***************************************************************************/

#include	"always.h"
#include	"plot_lib.h"
#include	"../main_src/diva.h"

	int		af_plot_flag = 0;
	area_function	af_pix[100];
	int		ns_pix;

/*****
*	Function: read_af
*	Note :	Read area function data (for vocal trac and nasal tract )
*		from a text file.
*****/
void	read_af (
	char	*af_path,		/* complete file specification	*/
	int	*nss,			/* # of sections		*/
	area_function	**af )		/* area function		*/
{
	FILE	*in;
	area_function	*p;
	char	af_title[6];
	double	a, x;

	in = fopen(af_path, "rt");
	fscanf(in, "%s", af_title );
	fscanf(in, "%d", nss);
	*af = (area_function *) calloc( *nss, sizeof(area_function) );
	fscanf(in, "%lf", &x);
	for(p = *af; p<*af+*nss; p++)
	{  fscanf(in, "%lf", &a);
	   p->A = a;
	   p->x = x;
	}
	fclose ( in );
}


/*****
*       Function : plot_af
*	Note	 : Plot the area function of the vocal tract
*****/

void	plot_af ( int ns, area_function *af )
{
	double  x1=0.0,x2=0.0;
	double  y1=0.0,y2=0.0;
	double  x_offset=400.0, y_offset=600.0;
	int	i;

	for( i=0; i<ns; i++)
	{
		af_pix[i].x = af[i].x;
		af_pix[i].A = af[i].A;
	}
	ns_pix = ns;

	for( i=0; i<ns; i++)
	{
#ifdef COMMENT
	   printf("i=%d, af[i].x= %f, af[i].A= %f\n", i, af[i].x, af[i].A);
#endif COMMENT

	   x1 = x2;
	   x2 = x1 + af[i].x;
	   y1 = y2;
	   y2 = af[i].A;

	   MyDrawLine(theDisplay, theWindow, theGC,
		(double) 10.*x1 + x_offset,
		(double) 50.-5.*y1 + y_offset,
		(double) 10.*x1 + x_offset,
		(double) 50.-5.*y2 + y_offset);
	   MyDrawLine(theDisplay, theWindow, theGC,
		(double) 10.*x1 + x_offset,
		(double) 50.-5.*y2 + y_offset,
		(double) 10.*x2 + x_offset,
		(double) 50.-5.*y2 + y_offset);
	}
	MyDrawLine(theDisplay, theWindow, theGC,
		(double) 10.*x2 + x_offset,
		(double) 50.-5.*y2 + y_offset,
		(double) 10.*x2 + x_offset,
		(double) 50.-0. + y_offset);
	af_plot_flag = 1;
}

/*****
*       Function : deplot_af
*	Note	 : unPlot the area function of the vocal tract
*****/

void	deplot_af ( void )
{
	double  x1=0.0,x2=0.0;
	double  y1=0.0,y2=0.0;
	double  x_offset=400.0, y_offset=600.0;
	int	i;

	if (af_plot_flag == 0) return;
	XSetForeground(theDisplay,theGC,theWhitePixel);

	for( i=0; i<ns_pix; i++)
	{
#ifdef COMMENT
	   printf("i=%d, af[i].x= %f, af[i].A= %f\n", i, af[i].x, af[i].A);
#endif COMMENT

	   x1 = x2;
	   x2 = x1 + af_pix[i].x;
	   y1 = y2;
	   y2 = af_pix[i].A;

	   MyDrawLine(theDisplay, theWindow, theGC,
		(double) 10.*x1 + x_offset,
		(double) 50.-5.*y1 + y_offset,
		(double) 10.*x1 + x_offset,
		(double) 50.-5.*y2 + y_offset);
	   MyDrawLine(theDisplay, theWindow, theGC,
		(double) 10.*x1 + x_offset,
		(double) 50.-5.*y2 + y_offset,
		(double) 10.*x2 + x_offset,
		(double) 50.-5.*y2 + y_offset);
	}
	MyDrawLine(theDisplay, theWindow, theGC,
		(double) 10.*x2 + x_offset,
		(double) 50.-5.*y2 + y_offset,
		(double) 10.*x2 + x_offset,
		(double) 50.-0. + y_offset);
	XSetForeground(theDisplay,theGC,theBlackPixel);
}

/*****
*       Function : plot_af_source_loc
*	Note	 : Plot the area function of the vocal tract and a source
*		   location on a predefined frame and a viewport ( vp.n ).
*****/

#ifdef COMMENT
void	plot_af_source_loc (
	char		*title,		/* main title			*/
	int		srcloc,		/* source location in section #	*/
	int		ns,		/* # of sections		*/
	area_function	*af,		/* area function		*/
	vp_lo_frame	vp )		/* viewport specification	*/
{
	int	i;
	double	*A_plt, *x_plt, x;

	def_vp(vp.n, vp.x, vp.y, vp.x + vp.w, vp.y + vp.h, 1);
	open_vp(vp.n);
	clear_vp(vp.n);
	g_frame ( vp.n,
		  1, title, "Distance from glottis (cm)",
		  "Area (cm~2)",
		  0.,  15.,  19., 0.0,
		  0, 5, 0.0,  1.0, 19.,
		  0, 2, 0.0,  1.0, 15.0 );

	A_plt = (double *) calloc(2*ns, sizeof(double));
	x_plt = (double *) calloc(2*ns, sizeof(double));
	x = 0;
	for( i=0; i<ns; i++)
	{  A_plt[2*i] = (double) max( 0., af[i].A);
	   x_plt[2*i] = x;
	   x = x + af[i].x;
	   A_plt[2*i + 1] = (double) max(0., af[i].A);
	   x_plt[2*i + 1] = x;
	}
	plotdata_xy(vp.n, 2*ns, x_plt, A_plt);
/* put a mark at the source location */
	i = max( 0, 2*srcloc - 1);
	diamond_lo( datolo_x( vp.n, x_plt[i] ),
		    datolo_y( vp.n, A_plt[i]/2.), 1.0 );
	free ( A_plt );
	free ( x_plt );
}
#endif COMMENT

/*****
*	Function : plot_signal
*	Note :	Plot signal on a fixed format graphics frame, i.e.,
*		time (ms) vs. relative pressure.
*****/
#ifdef COMMENT
void	plot_signal (
	int	n,		/* number of samples */
	double	*t,		/* time coordinate value in ms */
	double	*s,		/* signal */
	vp_lo_frame vp )	/* frame in logical units */
{
	double	tmax;

	tmax = t[n-1];
	def_vp(vp.n, vp.x, vp.y, vp.x+vp.w, vp.y+vp.h, 1);
	clear_vp( vp.n );
	open_vp(vp.n);
	g_frame (vp.n,
	   1, "Time response", "time (ms)", "pressure (relative)",
	   0., 20., tmax, -20.,
	   0, 1, 0., 10., tmax,
	   0, 1, -20., 5., 20.);
	plotdata_xy(vp.n, n, t, s);
}
#endif COMMENT

/*****
*	function : plot_tf
*	Note :	Plot VT transfer function
****/
#ifdef COMMENT
void	plot_tf (
	int	n,		/* number of spectral samples */
	double	*f,		/* frequencies in Hz */
	double	*tf,		/* transfer magnitude in dB */
	vp_lo_frame vp )	/* viewport spec */
{
	double	fmax;

	fmax = f[n-1];
	def_vp(vp.n, vp.x, vp.y, vp.x+vp.w, vp.y+vp.h, 1);
	open_vp(vp.n);
	clear_vp(vp.n);
	g_frame (vp.n,
	   1, "Vocal tract transfer (U/Ug)", "frequency (Hz)",
	      "magnitude (dB)",
	   0., 40., fmax, -40.0,
	   0, 1, 0., 1000., fmax,
	   0, 1, -40., 10., 40.);
	plotdata_xy(vp.n, n, f, tf);
}
#endif COMMENT

/******
*	Function : plot_FA
*	Note :	Plot formant Frequencie and Amplitude values
*		in graphics mode.
******/
#ifdef COMMENT
void	plot_FA (
	int	nfrms,		/* number of formants */
	double	*frm,		/* formant frequencies in Hz */
	double	*amp,		/* formant amplitudes in dB */
	vp_lo_frame vp )	/* view port spec. */
{
	int	i;
	char	txt[40];

	def_vp(vp.n, vp.x, vp.y, vp.x + vp.w, vp.y + vp.h, 1);
	open_vp(vp.n);
	clear_vp(vp.n);
	plot_vp(vp.n);
	sprintf(txt, "  Frq(Hz) A(dB)");
	plottextxy( 1., 1., txt );
	for(i=0; i<nfrms; i++)
	{  sprintf(txt, "F%1d %5d %5d",
			i+1, (int)(frm[i]+.5), (int)(amp[i]+.5) );
	   plottextxy( 1., 2.5*i + 4., txt);
	}
}
#endif COMMENT


/******
*	Function : plot_FBA
*	Note :	Plot formant Frequencie, Bandwidth and Amplitude values
*		in graphics mode.
******/
#ifdef COMMENT
void	plot_FBA (
	int	nfrms,		/* number of formants */
	double	*frm,		/* formant frequencies in Hz */
	double	*bw,		/* formant bandwidth in Hz */
	double	*amp,		/* formant amplitudes in dB */
	vp_lo_frame vp )	/* view port spec. */
{
	int	i;
	char	txt[40];

	def_vp(vp.n, vp.x, vp.y, vp.x + vp.w, vp.y + vp.h, 1);
	open_vp(vp.n);
	clear_vp(vp.n);
	plot_vp(vp.n);
	sprintf(txt, "  Frq(Hz) Bw(Hz) A(dB)");
	plottextxy( 1., 1., txt );
	for(i=0; i<nfrms; i++)
	{  if( bw[i] > 0. )
	      sprintf( txt, "F%1d %5d %5d %5d", i+1,
		      (int)(frm[i]+.5), (int)(bw[i]+.5), (int)(amp[i]+.5) );
	   else
	      sprintf( txt, "F%1d %5d     * %5d", i+1,
		      (int)(frm[i]+.5), (int)(amp[i]+.5) );
	   plottextxy( 1., 2.5*i + 4., txt);
	}
}
#endif COMMENT


/******
*	Function : plot_PUR
*	Note :	Plot DC air flow, and resistances in graphics mode.
******/
#ifdef COMMENT
void	plot_PUR (
	double	Psub,		/* Subglottal air pressure in cmH2O */
	double	Udc,		/* air flow in cm3/s */
	double	Rg_v,		/* in acoustic ohm */
	double	Rg_k,		/* in acoustic ohm */
	double	Rc_k,		/* in acoustic ohm */
	vp_lo_frame vp )	/* view port spec. */
{
	char	txt[40];

	def_vp(vp.n, vp.x, vp.y, vp.x + vp.w, vp.y + vp.h, 1);
	open_vp(vp.n);
	clear_vp(vp.n);
	plot_vp(vp.n);

	sprintf( txt, "Psub = %8.1f cmH2O", Psub );
	plottextxy ( 1., 1.0, txt);
	sprintf( txt, "Udc  = %8.1f cm3/s", Udc );
	plottextRxy( 0., 2.5, txt);
	sprintf( txt, "Rg_v = %8.1f Ohms", Rg_v );
	plottextRxy( 0., 2.5, txt);
	sprintf( txt, "Rg_k = %8.1f Ohms", 2.*Rg_k );
	plottextRxy( 0., 2.5, txt);
	sprintf( txt, "Rc_k = %8.1f Ohms", 2.*Rc_k );
	plottextRxy( 0., 2.5, txt);

}
#endif COMMENT

/*****
*	Function : plot_glottal_area
*	Note :	Plot glottal area variation on a fixed format graphics
*		frame, i.e., time (ms) vs. area (cm2).
*****/
#ifdef COMMENT
void	plot_glottal_area (
	int	n,		/* number of samples */
	double	tmin,		/* left time coordinate in ms */
	double	tmax,		/* right time coordinate in ms */
	double	*t,		/* time coordinate value in ms */
	double	*s,		/* glottal area (cm2) */
	vp_lo_frame vp )	/* frame in logical units */
{
	def_vp(vp.n, vp.x, vp.y, vp.x + vp.w, vp.y + vp.h, 1);
	clear_vp( vp.n );
	open_vp( vp.n );
	g_frame (vp.n,
	   1, "Glottal area", "time (ms)", "area (cm2)",
	   tmin, 0.3, tmax, 0.,
	   0, 1, tmin, 10., tmax,
	   0, 1, 0.0, 0.1, 0.3);
	plotdata_xy(vp.n, n, t, s);
}
#endif COMMENT


/*****
*	Function : plot_speech_signal
*	Note :	Plot speech signal on a fixed format graphics
*		frame, i.e., time (ms) vs. relative pressure.
*****/
#ifdef COMMENT
void	plot_speech_signal (
	int	n,		/* number of samples */
	double	tmin,		/* left time coordinate in ms */
	double	tmax,		/* right time coordinate in ms */
	double	*t,		/* time coordinate value in ms */
	double	*s,		/* glottal area (cm2) */
	vp_lo_frame vp )	/* frame in logical units */
{
	def_vp(vp.n, vp.x, vp.y, vp.x + vp.w, vp.y + vp.h, 1);
	clear_vp( vp.n );
	open_vp( vp.n );
	g_frame (vp.n,
	   1, "Speech signal", "time (ms)", "pressure (relative)",
	   tmin, 3., tmax, -3.,
	   0, 1, tmin, 10., tmax,
	   0, 1, -3., 1., 3.);
	plotdata_xy(vp.n, n, t, s);
}
#endif COMMENT
