/******
*	File :	vtconfig.h
*	Note :	Acoustic and geometrical configurations for the vocal
*		tract calculations.  It should be declared in "main".
******/

#include	"vtsimul.h"

/******************( Global variables defined in "main" )****************/


	/* Specific to time domain calculations */

double	simfrq = 30000.0;	/* simulation frequency in Hz	*/
double	smpfrq = 10000.0;	/* sampling freq. in Hz		*/

	/*( Used in the time and frequency domain calculations )*/

double	Psub = 8.;		/* subglottal air pressure in cmH2O	*/
double	Ag = 0.0;		/* glottis area in cm**2		*/
double	xg = 0.3;		/* glottis thickness in cm (=0.3 cm)	*/
double	lg = 1.2;		/* fold length in cm			*/
double	Kc = 1.42;		/* Ishizaka's for turbulent flow	*/
				/* =.875, Van der Berg's constant	*/

int	nbu = 8;		/* # of sections in the bucal tube	*/
int	nph = 9;		/* # of sections in the phryngeal tube	*/
int	nvt;			/* = nbu + nph, # of VT sections	*/
area_function	*afvt;		/* AF from the glottis to the lips	*/

double	anc = 0.;		/* nasal coupling area in cm2		*/
int	nna;			/* # of sections in the nasal tract	*/
area_function	*afnt;		/* AF from nostrils to coupling point	*/

int	nss;			/* total # of sections in VT system	*/

/************************( simulation options )**************************/

int	nasal_tract  = OFF;		/* OFF or ON			*/
int	wall         = RIGID;		/* YIELDING or RIGID		*/
int	rad_boundary = SHORT_CIRCUIT;	/* SHORT_CIRCUIT, or BESSEL_FUN	*/
					/* or RL_CIRCUIT		*/
int	glt_boundary = CLOSE;		/* or OPEN			*/

int	source_loc = 0;		/* source location in VT section number	*/
int	source_typ = FLOW;		/* or PRESSURE			*/

	/* specific to time_domain calculations */

int	vocal_tract  = STATIONARY;	/* or TIME_VARYING		*/
int	dynamic_term = OFF;		/* or ON			*/

/************( an extra heat loss factor for the nasal tract )***********/

double	extra_loss_factor = 50.;

/************************( physical constants )**************************/

double	ro    = 1.14e-3;	/* air density, gm/cm**3		*/
double	c     = 3.5e+4;		/* sound velocity, cm/s			*/
double	eta   = 1.4;		/* adiabatic constant			*/
double	cp    = 0.24;		/* specific heat, cal/gm.degree		*/
double	lamda = 5.5e-5;		/* heat conduction, cal/cm.sec.degree	*/
double	mu    = 1.86e-4;	/* viscosity coef, dyne.sec/cm**2	*/
double	wall_resi = 1600.;	/* wall resistance, gm/s/cm2		*/
double	wall_mass = 1.5;	/* wall mass per area, gm/cm2		*/
double	wall_comp = 3.0e+5;	/* wall compliance			*/
double	H2O_bar= 980.39;
