/***************************************************************************
*									   *
*	File :	frm_plot.c						   *
*	Note :	plotting functions for plotting formants in main window	   *
*									   *
***************************************************************************/

#include	"../maeda_code/always.h"
#include	"../maeda_code/plot_lib.h"
#include	"diva.h"

void draw_frm_dashed_lines( void );
void frm_replot( void );

#define X_PIX_MAX 500
#define SHIFT_AMOUNT 15

typedef struct {
	double x;
	double y;
	} FRM_XY_PIX;

FRM_XY_PIX frm_xy_array[500][3];

double	y_grid;
double	x_pix_min=10.0, y_pix_min=0.0;
double	x_pix_max=(double)X_PIX_MAX, y_pix_max=130.0;
double	x_range=500.0, y_range=4000.0;
int	frm_stored_count=0; /* # of line segments stored per formant */

/*****
*       Function : frm_plot
*	Note	 : plot F1, F2, F3
*****/
void	frm_plot ( double *frm )
{
	static	int	x=999;
	static	double	frm_old[3]={0.0, 0.0, 0.0};
	static	double	frm_pix_old[3]={0.0, 0.0, 0.0};
	static	double	x_pix, frm_pix[3], x_pix_old;

	static	int	first_time_called_flag=1;

	int	i,j;
	int	del_x=4;

	/* at the right-most boundary of plotting window, reset */
	if (x > x_range) {

		if (first_time_called_flag) {

			x_pix_old = x_pix_min;
			x = del_x;
			frm_stored_count=0;

			/* set frm_pix_old correctly */
			for (i=0; i<3; i++) {
				frm_pix_old[i] =
					0.0-y_pix_max/y_range*(frm[i]-y_range);
				if (frm_pix_old[i] > y_pix_max)
					frm_pix_old[i] = y_pix_max; 
			}

			for (i=0; i<3; i++) {
				frm_xy_array[0][i].x = x_pix_old;
				frm_xy_array[0][i].y = frm_pix_old[i];
			}

			first_time_called_flag=0;
			draw_frm_dashed_lines();
		}

		else {
			/* shift the display left some # of line segments */
			frm_stored_count -= SHIFT_AMOUNT;
			for (i=0; i<3; i++)
			    for (j=0; j<frm_stored_count+1; j++)
				frm_xy_array[j][i].y =
					frm_xy_array[j+SHIFT_AMOUNT][i].y;

			x_pix_old = frm_xy_array[frm_stored_count-1][i].x;
			x = del_x * (frm_stored_count+1);
			frm_replot();
		}
	}

	x_pix = x_pix_min + (x_pix_max - x_pix_min)/x_range*x;

	/* compute pixels corresponding to next formant */
	for (i=0; i<3; i++) {
		frm_pix[i] = 0.0 - y_pix_max/y_range*(frm[i] - y_range);
		if (frm_pix[i] > y_pix_max) frm_pix[i] = y_pix_max; 
	}

	frm_stored_count++; /* # frm line segments stored */
	for (i=0; i<3; i++) {
		frm_xy_array[frm_stored_count][i].x = x_pix;
		frm_xy_array[frm_stored_count][i].y = frm_pix[i];
	}

	/* plot next formant */
	for (i=0; i<3; i++) {
		MyDrawLine(theDisplay, theWindow, theGC,
			(double) x_pix_old,
			(double) frm_pix_old[i],
			(double) x_pix,
			(double) frm_pix[i]);
	}

	for (i=0; i<3; i++)
		frm_pix_old[i] = frm_pix[i];
	x_pix_old = x_pix;
	x += del_x;
}


/* This function must not be called before the first call to frm_plot() */
void frm_replot( void )
{
	int i,j;

	draw_frm_dashed_lines();

	/* draw any old samples */
	for (i=0; i<3; i++)
		for (j=0; j<frm_stored_count; j++) {
			MyDrawLine(theDisplay, theWindow, theGC,
				(double) frm_xy_array[j][i].x,
				(double) frm_xy_array[j][i].y,
				(double) frm_xy_array[j+1][i].x,
				(double) frm_xy_array[j+1][i].y);
		}
}


void draw_frm_dashed_lines( void )
{
	int	i;

	XSetForeground(theDisplay,theGC,theWhitePixel);
	MyFillRectangle(theDisplay, theWindow, theGC,
			(double) x_pix_min,
			(double) y_pix_min,
			(double) x_pix_max - x_pix_min + 1,
			(double) y_pix_max - y_pix_min + 1);
	XSetForeground(theDisplay,theGC,theBlackPixel);

	/* horizontal dashed lines at 0, 1000, 2000, 3000 Hz */
	XSetLineAttributes(theDisplay,theGC,
			0, LineOnOffDash, CapButt, JoinRound);
	for (i=0; i<5; i++) {
		y_grid = 0.0 - y_pix_max/y_range*(i*1000 - y_range);
		MyDrawLine(theDisplay, theWindow, theGC,
			(double) x_pix_min,
			(double) y_grid,
			(double) x_pix_max,
			(double) y_grid);
	}
	XSetLineAttributes(theDisplay,theGC,
			0, LineSolid, CapButt, JoinRound);
}
