#include "diva.h"
#include <math.h>

void name_phonemes(void);
void get_phonemes(void);

extern double get_min_area();
extern double get_SF0();

#ifdef COMMENT
int lip_aperture;
#endif COMMENT

void name_phonemes(void)
{
  /**** vowels -- see p. 18 of Flanagan "Speech Analysis" for real names ****/
    p_name[1] = 'E';  /** eve **/
    p_name[2] = 'i';  /** it **/
    p_name[3] = 'e';  /** met **/
    p_name[4] = 'a';  /** at **/
    p_name[5] = 'C';  /** father **/
    p_name[6] = 'c';  /** all **/
    p_name[7] = 'o';  /** foot **/
    p_name[8] = 'U';  /** boot **/
    p_name[9] = 'u';  /** up **/
    p_name[10] = 'I';  /** bird **/
    p_name[11] = 'F';  /** vous **/
    
  /**** not used ****/
/*    p_name[11] = 'O';*/  /** obey **/
    p_name[12] = 'A';  /** hate **/
  /**** fricative consonants - see p. 20 ****/
    p_name[13] = 'f';  /** for **/
    p_name[14] = 'T';  /** thin **/
    p_name[15] = 's';  /** see **/
    p_name[16] = 'S';  /** she **/
    p_name[17] = 'h';  /** he **/
    p_name[18] = 'v';  /** vote **/
    p_name[19] = 'D';  /** then **/
    p_name[20] = 'z';  /** zoo **/
    p_name[21] = 'Z';  /** azure **/
  /**** stop consonants - see p. 20 ****/
    p_name[22] = 'p';  /** pay **/
    p_name[23] = 't';  /** to **/
    p_name[24] = 'k';  /** key **/
    p_name[25] = 'b';  /** be **/
    p_name[26] = 'd';  /** day **/
    p_name[27] = 'g';  /** go **/
  /**** nasal consonants - see p. 21 ****/
    p_name[28] = 'm';  /** me **/
    p_name[29] = 'n';  /** no **/
    p_name[30] = 'N';  /** sing **/
  /**** semivowels (liquids) and glides - see p. 22 ****/
    p_name[31] = 'r';  /** read **/
    p_name[32] = 'l';  /** let **/
    /* glides are not used - use U and E instead */
/*  p_name[33] = 'w'; */ /** we **/
/*  p_name[34] = 'y'; */ /** you **/

}


/*******************************************************************
This function will look at the formant stored in syn_targets[] 
to determine if any phonemes are being produced.

WARNING: This function uses get_min_area(), so before calling it, the
area function must have been computed by a call to either find_area_function()
directly or indirectly through check_valid_configuration(). In either
case, AMpar will need to be set beforehand.
*********************************************************************/  

void get_phonemes(void)
{
    int i,j;
    double SR0;
    double ratio[NUM_TARGETS];
    static init_flag ;  /*static vars automatically init'd to 0 */
    FILE *ratios ;
   double x[NUM_STATE_VARS + 1],y[NUM_STATE_VARS + 1],w[NUM_STATE_VARS + 1];


#ifdef COMMENT
    lip_aperture = ARTIC_POS[4][1] - ARTIC_POS[5][1];
#endif COMMENT

    /*    find_area_function();*/
    SR0 = get_SF0(F0);  /* Assuming 100Hz is "ideal" F0 */

    for (i=1; i<=NUM_PHONEMES; i++)
	p[i]=0;
/*
    printf("f ");
    for(i=0;i<3;i++){
       printf("%lf ",syn_targets[i]);
    }
    printf("\n");
*/
    if (USE_RATIO_COORDS == 1)
      {
	 /* The current ratios being heard:  */
	 if(syn_targets[0]&&syn_targets[1]&&syn_targets[2])
	   {
	      ratio[0] = log10(syn_targets[0]/SR0) ;  
	      ratio[1] = log10(syn_targets[1]/syn_targets[0]) ; 
	      ratio[2] = log10(syn_targets[2]/syn_targets[1]) ; 
	   }else return;

	 /***********************************/
	 /****** Vowels  ******* ************/
	 /***********************************/

         for(i=1;i<=NUM_PHONEMES;i++)/*NUM_PHONEMES to be repl'd w NUM_VOWELS*/
	   {
            if((MIN_RATIO[i][0] < ratio[0])&&(MAX_RATIO[i][0] > ratio[0])&&
               (MIN_RATIO[i][1] < ratio[1])&&(MAX_RATIO[i][1] > ratio[1])&&
               (MIN_RATIO[i][2] < ratio[2])&&(MAX_RATIO[i][2] > ratio[2])&&
               (get_min_area() > 0.2) &&
	       (voicing == 1) && (noise == 0) &&  (pressure == 0) )
	     {
                p[i] = 1;
		if (i == 1)
		  {
		     normalize_ratios(x,&MIN_RATIO[i][0]);
		     normalize_ratios(y,&MAX_RATIO[i][0]);
		     normalize_ratios(w,ratio);
		  }
	     }
            else
	     {
                p[i] = 0;
             }
	 }
      }
    else  /* else use formant coords */
      {
	 /**** "E" as in eve (IY) ****/
	 if ( (220 < syn_targets[0]) && (syn_targets[0] < 320) &&
	     (2140 < syn_targets[1]) && (syn_targets[1] < 2440) &&
	     (2810 < syn_targets[2]) && (syn_targets[2] < 3510) &&
	     (get_min_area() > 0.2) &&
	     (voicing == 1) && (noise == 0) &&  (pressure == 0) )
	   {
	      p[1] = 1;
	   }
	 else
	   {
	      p[1] = 0;
	   }
 
	 /**** "i" as in it (I) ****/
	 if ( (340 < syn_targets[0]) && (syn_targets[0] < 440) &&
	     (1890 < syn_targets[1]) && (syn_targets[1] < 2090) &&
	     (2350 < syn_targets[2]) && (syn_targets[2] < 2950) &&
	     (get_min_area() > 0.2) &&
	     (voicing == 1) && (noise == 0) &&  (pressure == 0) )
	   {
	      p[2] = 1;
	   }
	 else
	   {
	      p[2] = 0;
	   }
	 

	 /**** "e" as in met (E) ****/
	 if ( (480 < syn_targets[0]) && (syn_targets[0] < 580) &&
	     (1740 < syn_targets[1]) && (syn_targets[1] < 1940) &&
	     (2280 < syn_targets[2]) && (syn_targets[2] < 2980) &&
	     (get_min_area() > 0.2) &&
	     (voicing == 1) && (noise == 0) &&  (pressure == 0) )
	   {
	      p[3] = 1;
	   }
	 else
	   {
	      p[3] = 0;
	   }

	 /**** "a" as in at (AE) ****/
	 if ( (610 < syn_targets[0]) && (syn_targets[0] < 710) &&
	     (1620 < syn_targets[1]) && (syn_targets[1] < 1820) &&
	     (2210 < syn_targets[2]) && (syn_targets[2] < 2910) &&
	     (get_min_area() > 0.2) &&
	     (voicing == 1) && (noise == 0) &&  (pressure == 0) )
	   {
	      p[4] = 1;
	   }
	 else
	   {
	      p[4] = 0;
	   }

	 /**** "C" as in father (A) ****/
	 if ( (630 < syn_targets[0]) && (syn_targets[0] < 810) &&
	     (990 < syn_targets[1]) && (syn_targets[1] < 1190) &&
	     (2240 < syn_targets[2]) && (syn_targets[2] < 2940) &&
	     (get_min_area() > 0.2) &&
	     (voicing == 1) && (noise == 0) &&  (pressure == 0) )
	   {
	      p[5] = 1;
	   }
	 else
	   {
	      p[5] = 0;
	   }


	 /**** "c" as in all (OW) ****/
	 if ( (520 < syn_targets[0]) && (syn_targets[0] < 620) &&
	     (740 < syn_targets[1]) && (syn_targets[1] < 940) &&
	     (2210 < syn_targets[2]) && (syn_targets[2] < 2910) &&
	     (get_min_area() > 0.2) &&
	     (voicing == 1) && (noise == 0) &&  (pressure == 0) )
	   {
	      p[6] = 1;
	   }
	 else
	   {
	      p[6] = 0;
	   }

	 /**** "o" as in foot (U) ****/
	 if ( (390 < syn_targets[0]) && (syn_targets[0] < 470) &&
	     (920 < syn_targets[1]) && (syn_targets[1] < 1120) &&
	     (2040 < syn_targets[2]) && (syn_targets[2] < 2740) &&
	     (get_min_area() > 0.2) &&
	     (voicing == 1) && (noise == 0) &&  (pressure == 0) )
	   {
	      p[7] = 1;
	   }
	 else
	   {
	      p[7] = 0;
	   }

	 /**** "U" as in boot (OO) ****/
	 if ( (250 < syn_targets[0]) && (syn_targets[0] < 350) &&
	     (770 < syn_targets[1]) && (syn_targets[1] < 970) &&
	     (2040 < syn_targets[2]) && (syn_targets[2] < 2740) &&
	     (get_min_area() > 0.1) &&
	     (voicing == 1) && (noise == 0) &&  (pressure == 0) )
	   {
	      p[8] = 1;
	   }
	 else
	   {
	      p[8] = 0;
	   }

	 /**** "u" as in up (UH) ****/
	 if ( (390 < syn_targets[0]) && (syn_targets[0] < 590) &&
	     (1090 < syn_targets[1]) && (syn_targets[1] < 1260) &&
	     (2190 < syn_targets[2]) && (syn_targets[2] < 2890) &&
	     (get_min_area() > 0.2) &&
	     (voicing == 1) && (noise == 0) &&  (pressure == 0) )
	   {
	      p[9] = 1;
	   }
	 else
	   {
	      p[9] = 0;
	   }

	 /**** "I" as in bird (ER) ****/
	 /* was 440 */
	 if ( (390 < syn_targets[0]) && (syn_targets[0] < 590) &&
	     (1280 < syn_targets[1]) && (syn_targets[1] < 1450) &&
	      (1590 < syn_targets[2]) && (syn_targets[2] < 1790) && 
	     (get_min_area() > 0.2) &&
	     (voicing == 1) && (noise == 0) &&  (pressure == 0) )
	   {
	      p[10] = 1;
	   }
	 else
	   {
	      p[10] = 0;
	   }


        /*******"F" as in vous   *************/

	 if ( (184 < syn_targets[0]) && (syn_targets[0] < 434) &&
	     (586 < syn_targets[1]) && (syn_targets[1] < 770) &&
	     (1854 < syn_targets[2]) && (syn_targets[2] < 2254) && 
	     (get_min_area() > 0.2) &&
	     (voicing == 1) && (noise == 0) &&  (pressure == 0) )
	   {
	      p[11] = 1;
	   }
	 else
	   {
	      p[11] = 0;
	   }

#ifdef COMMENT
	 /**** "O" as in obey ****/ ??????????????????
	   if ( (560 < syn_targets[0]) && (syn_targets[0] < 560) &&
	       (1560 < syn_targets[1]) && (syn_targets[1] < 1560) &&
	       (2560 < syn_targets[2]) && (syn_targets[2] < 2560) &&
	       (voicing == 1) && (noise == 0) &&  (pressure == 0) )
	     {
		p[7] = 1;
	     }
	   else
	     {
		p[7] = 0;
	     }

	 /**** "A" as in hate ****/
	 if ( (620 < syn_targets[0]) && (syn_targets[0] < 740) &&
	     (1620 < syn_targets[1]) && (syn_targets[1] < 1740) &&
	     (2620 < syn_targets[2]) && (syn_targets[2] < 2740) && 
	     (voicing == 1) && (noise == 0) &&  (pressure == 0) )
	   {
	      p[3] = 1;
	   }
	 else
	   {
	      p[3] = 0;
	   }
	 /***********************************/
	 /****** Fricative consonants *******/
	 /***********************************/
	 
	 /**** "f" as in for  ****/
	 if (((ARTIC_POS[2][0] < 340) && (ARTIC_POS[2][0] > 270)) &&
	     ((ARTIC_POS[2][1] < 500) && (ARTIC_POS[2][1] > 340)) &&
	     ((ARTIC_POS[3][0] < 400) && (ARTIC_POS[3][0] > 339)) &&
	     ((ARTIC_POS[3][1] < 500) && (ARTIC_POS[3][1] > 350)) &&
	     ((ARTIC_POS[4][1] < 350) && (ARTIC_POS[4][1] > 345)) &&
	     (velum_height <= 0) &&
	     ((lip_protrusion < 3) && (lip_protrusion > (-1))) &&
	     ((lip_aperture < 10) && (lip_aperture > 1)) &&
	     (voicing == 0))
	   {
	      p[13] = 1;
	   }
	 else
	   {
	      p[13] = 0;
	   }

	 /**** "T" as in thin  ****/
	 if (((ARTIC_POS[2][0] < 340) && (ARTIC_POS[2][0] > 270)) &&
	     ((ARTIC_POS[2][1] < 500) && (ARTIC_POS[2][1] > 350)) &&
	     (ARTIC_POS[3][0] > 430) &&
	     (velum_height <= 0) &&
	     (fabs(x_tongue_tip_y_location - (double)ARTIC_POS[3][1]) > 0.0) &&
	     (fabs(x_tongue_tip_y_location - (double)ARTIC_POS[3][1]) < 5.0) &&
	     ((lip_protrusion < 41) && (lip_protrusion > (-1))) &&
	     ((lip_aperture < 100) && (lip_aperture > 5)) &&
	     (voicing == 0))
	   {
	      p[14] = 1;
	   }
	 else
	   {
	      p[14] = 0;
	   }
	 
	 /**** "s" as in see  ****/
	 if (((ARTIC_POS[2][0] < 340) && (ARTIC_POS[2][0] > 270)) &&
	     ((ARTIC_POS[2][1] < 500) && (ARTIC_POS[2][1] > 340)) &&
	     ((ARTIC_POS[3][0] < 420) && (ARTIC_POS[3][0] > 380)) &&
	     (velum_height <= 0) &&
	     (fabs(x_tongue_tip_y_location - (double)ARTIC_POS[3][1]) > 0.0) &&
	     (fabs(x_tongue_tip_y_location - (double)ARTIC_POS[3][1]) < 5.0) &&
	     ((lip_protrusion < 41) && (lip_protrusion > (-1))) &&
	     ((lip_aperture < 50) && (lip_aperture > 5)) &&
	     (voicing == 0))
	   {
	      p[15] = 1;
	   }
	 else
	   {
	      p[15] = 0;
	   }
	 
	 /**** "S" as in she  ****/
	 if (((ARTIC_POS[2][0] < 390) && (ARTIC_POS[2][0] > 270)) &&
	     ((ARTIC_POS[2][1] < 500) && (ARTIC_POS[2][1] > 350)) &&
	     ((ARTIC_POS[3][0] < 380) && (ARTIC_POS[3][0] > 350)) &&
	     (velum_height <= 0) &&
	     (fabs(x_tongue_tip_y_location - (double)ARTIC_POS[3][1]) > 0.0) &&
	     (fabs(x_tongue_tip_y_location - (double)ARTIC_POS[3][1]) < 5.0) &&
	     ((lip_protrusion < 41) && (lip_protrusion > (-1))) &&
	     ((lip_aperture < 50) && (lip_aperture > 5)) &&
	     (voicing == 0))
	   {
	      p[16] = 1;
	   }
	 else
	   {
	      p[16] = 0;
	   }
	 
	 /**** "h" as in he  ****/
	 if (((ARTIC_POS[2][0] < 350) && (ARTIC_POS[2][0] > 310)) &&
	     ((ARTIC_POS[2][1] < 360) && (ARTIC_POS[2][1] > 340)) &&
	     ((ARTIC_POS[3][0] < 400) && (ARTIC_POS[3][0] > 350)) &&
	     ((ARTIC_POS[3][1] < 500) && (ARTIC_POS[3][1] > 360)) &&
	     (velum_height <= 0) &&
	     ((lip_protrusion < 30) && (lip_protrusion > 10)) &&
	     ((lip_aperture < 100) && (lip_aperture > 20)) &&
	     (voicing == 0) && (noise == 1) &&  (pressure == 0))
	   {
	      p[17] = 1;
	   }
	 else
	   {
	      p[17] = 0;
	   }
	 
	 /**** "v" as in vote  ****/
	 if (((ARTIC_POS[2][0] < 340) && (ARTIC_POS[2][0] > 270)) &&
	     ((ARTIC_POS[2][1] < 500) && (ARTIC_POS[2][1] > 340)) &&
	     ((ARTIC_POS[3][0] < 400) && (ARTIC_POS[3][0] > 339)) &&
	     ((ARTIC_POS[3][1] < 500) && (ARTIC_POS[3][1] > 350)) &&
	     ((ARTIC_POS[4][1] < 350) && (ARTIC_POS[4][1] > 345)) &&
	     (velum_height <= 0) &&
	     ((lip_protrusion < 3) && (lip_protrusion > (-1))) &&
	     ((lip_aperture < 10) && (lip_aperture > 1)) &&
	     (voicing == 1))
	   {
	      p[18] = 1;
	   }
	 else
	   {
	      p[18] = 0;
	   }
	 
	 /**** "D" as in then  ****/
	 if (((ARTIC_POS[2][0] < 340) && (ARTIC_POS[2][0] > 270)) &&
	     ((ARTIC_POS[2][1] < 500) && (ARTIC_POS[2][1] > 350)) &&
	     (ARTIC_POS[3][0] > 430) &&
	     (velum_height <= 0) &&
	     (fabs(x_tongue_tip_y_location - (double)ARTIC_POS[3][1]) > 0.0) &&
	     (fabs(x_tongue_tip_y_location - (double)ARTIC_POS[3][1]) < 10.0) &&
	     ((lip_protrusion < 25) && (lip_protrusion > (-1))) &&
	     ((lip_aperture < 100) && (lip_aperture > 5)) &&
	     (voicing == 1))
	   {
	      p[19] = 1;
	   }
	 else
	   {
	      p[19] = 0;
	   }
	 
	 /**** "z" as in zoo  ****/
	 
	 if (((ARTIC_POS[2][0] < 340) && (ARTIC_POS[2][0] > 270)) &&
	     ((ARTIC_POS[2][1] < 500) && (ARTIC_POS[2][1] > 340)) &&
	     ((ARTIC_POS[3][0] < 420) && (ARTIC_POS[3][0] > 390)) &&
	     (velum_height <= 0) &&
	     (fabs(x_tongue_tip_y_location - (double)ARTIC_POS[3][1]) > 0.0) &&
	     (fabs(x_tongue_tip_y_location - (double)ARTIC_POS[3][1]) < 5.0) &&
	     ((lip_protrusion < 41) && (lip_protrusion > (-1))) &&
	     ((lip_aperture < 50) && (lip_aperture > 5)) &&
	     (voicing == 1))
	   {
	      p[20] = 1;
	   }
	 else
	   {
	      p[20] = 0;
	   }
	 
	 /**** "Z" as in azure  ****/
	 if (((ARTIC_POS[2][0] < 390) && (ARTIC_POS[2][0] > 270)) &&
	     ((ARTIC_POS[2][1] < 500) && (ARTIC_POS[2][1] > 350)) &&
	     ((ARTIC_POS[3][0] < 390) && (ARTIC_POS[3][0] > 350)) &&
	     (velum_height <= 0) &&
	     (fabs(x_tongue_tip_y_location - (double)ARTIC_POS[3][1]) > 0.0) &&
	     (fabs(x_tongue_tip_y_location - (double)ARTIC_POS[3][1]) < 5.0) &&
	     ((lip_protrusion < 41) && (lip_protrusion > (-1))) &&
	     ((lip_aperture < 50) && (lip_aperture > 5)) &&
	     (voicing == 1))
	   {
	      p[21] = 1;
	   }
	 else
	   {
	      p[21] = 0;
	   }
	 
	 /***********************************/
	 /****** Stop consonants ************/
	 /***********************************/
	 
	 /**** "p" as in pay  ****/
	 if ((ARTIC_POS[2][1] > 320) && 
	     (ARTIC_POS[3][1] > 320) &&
	     /*
		((ARTIC_POS[2][0] < 340) && (ARTIC_POS[2][0] > 270)) &&
		((ARTIC_POS[2][1] < 500) && (ARTIC_POS[2][1] > 320)) &&
		((ARTIC_POS[3][0] < 451) && (ARTIC_POS[3][0] > 339)) &&
		((ARTIC_POS[3][1] < 500) && (ARTIC_POS[3][1] > 320)) &&
		*/
	     (velum_height <= 0) &&
	     (fabs(x_tongue_tip_y_location - ARTIC_POS[3][1] < 0.00001)) &&
	     ((lip_protrusion < 40) && (lip_protrusion > 5)) &&
	     (fabs(x_lower_lip_location - ARTIC_POS[4][1]) > 5.0) &&
	     /* (fabs(x_lower_lip_location - ARTIC_POS[4][1]) < 7.0) &&  */
	     (fabs(x_upper_lip_location - ARTIC_POS[5][1]) > 5.0) &&
	     /* (fabs(x_upper_lip_location - ARTIC_POS[5][1]) < 7.0) && */
	     (voicing == 0) && (noise == 0) &&  (pressure == 1))
	   {
	      p[22] = 1;
	   }
	 else
	   {
	      p[22] = 0;
	   }
	 
	 /**** "t" as in to  ****/
	 if (((ARTIC_POS[2][0] < 340) && (ARTIC_POS[2][0] > 270)) &&
	     ((ARTIC_POS[2][1] < 500) && (ARTIC_POS[2][1] > 350)) &&
	     (ARTIC_POS[3][0] > 430) &&
	     (velum_height <= 0) &&
	     (fabs(x_tongue_tip_y_location - (double)ARTIC_POS[3][1]) > 10.0) &&
	     ((lip_protrusion < 25) && (lip_protrusion > (-1))) &&
	     ((lip_aperture < 100) && (lip_aperture > 5)) &&
	     (voicing == 0) && (noise == 0) &&  (pressure == 1))
	   {
	      p[23] = 1;
	   }
	 else
	   {
	      p[23] = 0;
	   }
	 
	 /**** "k" as in key  ****/
	 if (((ARTIC_POS[2][0] < 400) && (ARTIC_POS[2][0] > 269)) &&
	     (fabs(x_tongue_body_y_location - (double)ARTIC_POS[2][1]) > 10.0) &&
	     ((ARTIC_POS[3][0] < 451) && (ARTIC_POS[3][0] > 339)) &&
	     ((ARTIC_POS[3][1] <  500) && (ARTIC_POS[3][1] > 350)) &&
	     (velum_height <= 0) &&
	     ((lip_protrusion < 40) && (lip_protrusion > (-1))) &&
	     ((lip_aperture < 140) && (lip_aperture > 5)) &&
	     (voicing == 0) && (noise == 0) &&  (pressure == 1))
	   {
	      p[24] = 1;
	   }
	 else
	   {
	      p[24] = 0;
	   }
	 
	 /**** "b" as in be  ****/
	 if ((ARTIC_POS[2][1] > 320) && 
	     (ARTIC_POS[3][1] > 320) &&
	     /*
		((ARTIC_POS[2][0] < 340) && (ARTIC_POS[2][0] > 270)) &&
		((ARTIC_POS[2][1] < 500) && (ARTIC_POS[2][1] > 320)) &&
		((ARTIC_POS[3][0] < 451) && (ARTIC_POS[3][0] > 339)) &&
		((ARTIC_POS[3][1] < 500) && (ARTIC_POS[3][1] > 320)) &&
		*/
	     (velum_height <= 0) &&
	     ((lip_protrusion < 40) && (lip_protrusion > 5)) &&
	     (fabs(x_lower_lip_location - ARTIC_POS[4][1]) > 5.0) &&
	     (fabs(x_upper_lip_location - ARTIC_POS[5][1]) > 5.0) &&
	     (voicing == 1) && (noise == 0) &&  (pressure == 1))
	   {
	      p[25] = 1;
	   }
	 else
	   {
	      p[25] = 0;
	   }
	 
	 /**** "d" as in day  ****/
	 if (((ARTIC_POS[2][0] < 340) && (ARTIC_POS[2][0] > 270)) &&
	     ((ARTIC_POS[2][1] < 500) && (ARTIC_POS[2][1] > 350)) &&
	     (ARTIC_POS[3][0] > 430) &&
	     (velum_height <= 0) &&
	     (fabs(x_tongue_tip_y_location - (double)ARTIC_POS[3][1]) > 10.0) &&
	     ((lip_protrusion < 41) && (lip_protrusion > (-1))) &&
	     ((lip_aperture < 100) && (lip_aperture > 5)) &&
	     (voicing == 1) && (noise == 0) &&  (pressure == 1))
	   {
	      p[26] = 1;
	   }
	 else
	   {
	      p[26] = 0;
	   }
	 
	 /**** "g" as in go  ****/
	 if (((ARTIC_POS[2][0] < 400) && (ARTIC_POS[2][0] > 269)) &&
	     (fabs(x_tongue_body_y_location - (double)ARTIC_POS[2][1]) > 10.0) &&
	     ((ARTIC_POS[3][0] < 451) && (ARTIC_POS[3][0] > 339)) &&
	     ((ARTIC_POS[3][1] <  500) && (ARTIC_POS[3][1] > 350)) &&
	     (velum_height <= 0) &&
	     ((lip_protrusion < 40) && (lip_protrusion > (-1))) &&
	     ((lip_aperture < 140) && (lip_aperture > 5)) &&
	     (voicing == 1) && (noise == 0) &&  (pressure == 1))
	   {
	      p[27] = 1;
	   }
	 else
	   {
	      p[27] = 0;
	   }
	 
	 /***********************************/
	 /****** Nasal consonants ***********/
	 /***********************************/
	 
	 /**** "m" as in me  ****/
	 if (((ARTIC_POS[2][0] < 340) && (ARTIC_POS[2][0] > 270)) &&
	     ((ARTIC_POS[2][1] < 350) && (ARTIC_POS[2][1] > 310)) &&
	     ((ARTIC_POS[3][0] < 450) && (ARTIC_POS[3][0] > 339)) &&
	     ((ARTIC_POS[3][1] < 500) && (ARTIC_POS[3][1] > 350)) &&
	     (velum_height > 30) &&
	     ((lip_protrusion < 41) && (lip_protrusion > (-1))) &&
	     (lip_aperture < 2) && 
	     (voicing == 1) && (noise == 0) &&  (pressure == 0))
	   {
	      p[28] = 1;
	   }
	 else
	   {
	      p[28] = 0;
	   }
	 
	 /**** "n" as in no  ****/
	 if (((ARTIC_POS[2][0] < 390) && (ARTIC_POS[2][0] > 270)) &&
	     ((ARTIC_POS[2][1] < 500) && (ARTIC_POS[2][1] > 350)) &&
	     (velum_height > 30) &&
	     (tip_dental_stop == 1) &&
	     ((lip_protrusion < 41) && (lip_protrusion > (-1))) &&
	     ((lip_aperture < 100) && (lip_aperture > 5)) &&
	     (voicing == 1) && (noise == 0) &&  (pressure == 0))
	   {
	      p[29] = 1;
	   }
	 else
	   {
	      p[29] = 0;
	   }
	 
	 /**** "N" as in sing  ****/
	 if (((ARTIC_POS[2][0] < 300) && (ARTIC_POS[2][0] > 270)) &&
	     (ARTIC_POS[2][1] < 302) && 
	     ((ARTIC_POS[3][0] < 451) && (ARTIC_POS[3][0] > 339)) &&
	     ((ARTIC_POS[3][1] < 500) && (ARTIC_POS[3][1] > 350)) &&
	     (velum_height > 30) &&
	     ((lip_protrusion < 41) && (lip_protrusion > (-1))) &&
	     ((lip_aperture < 100) && (lip_aperture > 5)) &&
	     (voicing == 1) && (noise == 0) &&  (pressure == 0))
	   {
	      p[30] = 1;
	   }
	 else
	   {
	      p[30] = 0;
	   }
	 
	 /***********************************/
	 /** Liquids (semivowels) && glides **/
	 /***********************************/
	 
	 /**** "r" as in read  ****/
	 if (((ARTIC_POS[2][0] < 340) && (ARTIC_POS[2][0] > 270)) &&
	     ((ARTIC_POS[2][1] < 500) && (ARTIC_POS[2][1] > 340)) &&
	     ((ARTIC_POS[3][0] < 390) && (ARTIC_POS[3][0] > 370)) &&
	     ((ARTIC_POS[3][1] < 350) && (ARTIC_POS[3][1] > 305)) &&
	     (velum_height <= 0) &&
	     ((lip_protrusion < 41) && (lip_protrusion > (-1))) &&
	     ((lip_aperture < 75) && (lip_aperture > 5)) &&
	     (voicing == 1) && (noise == 0) &&  (pressure == 0))
	   {
	      p[31] = 1;
	   }
	 else
	   {
	      p[31] = 0;
	   }
	 
	 /**** "l" as in let  ****/
	 if (((ARTIC_POS[2][0] < 340) && (ARTIC_POS[2][0] > 270)) &&
	     ((ARTIC_POS[2][1] < 500) && (ARTIC_POS[2][1] > 340)) &&
	     ((ARTIC_POS[3][0] < 450) && (ARTIC_POS[3][0] > 390)) &&
	     ((ARTIC_POS[3][1] < 315) && (ARTIC_POS[3][1] > 301)) &&
	     (velum_height <= 0) &&
	     ((lip_protrusion < 40) && (lip_protrusion > (-1))) &&
	     ((lip_aperture < 75) && (lip_aperture > 5)) &&
	     (voicing == 1) && (noise == 0) &&  (pressure == 0))
	   {
	      p[32] = 1;
	   }
	 else
	   {
	      p[32] = 0;
	   }
	 
	 
	 /**** "w" as in we  ****/
	 /* NOT USED - USE "U" AS IN boot */
	 /*
	    if (((ARTIC_POS[2][0] < 285) && (ARTIC_POS[2][0] > 275)) &&
	    ((ARTIC_POS[2][1] < 320) && (ARTIC_POS[2][1] > 310)) &&
	    ((ARTIC_POS[3][0] < 370) && (ARTIC_POS[3][0] > 340)) &&
	    ((ARTIC_POS[3][1] < 500) && (ARTIC_POS[3][1] > 360)) &&
	    (velum_height <= 0) &&
	    ((lip_protrusion < 40) && (lip_protrusion > 30)) &&
	    ((lip_aperture < 35) && (lip_aperture > 5)) &&
	    (voicing == 1) && (noise == 0) &&  (pressure == 0))
	    {
	    p[33] = 1;
	    }
	    else
	    {
	    p[33] = 0;
	    }
	    */
	 /**** "y" as in you  ****/
	 /* NOT USED - USE "E" AS IN eve */
	 /*
	    if (((ARTIC_POS[2][0] < 365) && (ARTIC_POS[2][0] > 355)) &&
	    ((ARTIC_POS[2][1] < 320) && (ARTIC_POS[2][1] > 310)) &&
	    ((ARTIC_POS[3][0] < 450  ) && (ARTIC_POS[3][0] > 420)) &&
	    ((ARTIC_POS[3][1] < 500) && (ARTIC_POS[3][1] > 360)) &&
	    (velum_height <= 0) &&
	    ((lip_protrusion < 20) && (lip_protrusion > 0)) &&
	    ((lip_aperture < 35) && (lip_aperture > 5)) &&
	    (voicing == 1) && (noise == 0) &&  (pressure == 0))
	    {
	    p[34] = 1;
	    }
	    else
	    {
	    p[34] = 0;
	    }
	    */
#endif COMMENT
      }
 }
