#include "diva.h"
#include "babble.h"
#include "../maeda_code/plot_lib.h"
#include "../rbf_src/rbf.h"
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

void apply_methods(void);
void get_methods(void);
void Mget_methods();
int get_formants(void);
int get_magnitudes(void);
double get_min_area( void );
int get_tessellated_bin(void);
void adjust_formants();

extern int bin[7];
extern area_function *afvt;
extern int nss;
extern int babbling_flag;

int valid_configuration=0;
double rate;


/*
 * SPEAKING_RATE may be set in the user interface.  We allow this to 
 * affect the rate variable when not babbling.  Several mappings between
 * SPEAKING_RATE and rate are available.
 *
 * This function returns the rate and also sets the global rate variable.
 */
double get_rate(void)
{
	if (babbling_flag)
		return(rate);
	else {
         	if(RBF) rate = (double)SPEAKING_RATE/100.0; 
                else    rate = (double)SPEAKING_RATE/100.0;
		return(rate);
	}


#ifdef COMMENT
    rate = (((double)SPEAKING_RATE + 200.0) / 300.0) + 0.001;

    rate = 1 - exp(-((double)(SPEAKING_RATE)/33.33));
#endif COMMENT
     }


void get_methods(void)
{
    int i,j;

    if(RBF == 0){

     get_tessellated_bin();

#ifdef COMMENT
    if (!babbling_flag) print_bin();
#endif COMMENT

     for (i = 1; i <= NUM_METHODS; i++) {
       m[i] = 0.0;
       for (j = 1; j <= NUM_STATE_VARS; j++) {

	 if ( (x[j] *
           zxm[bin[0]][bin[1]][bin[2]][bin[3]][bin[4]][bin[5]][bin[6]][j][i]) >
		0.0  &&  x[j] > 0.0001) /* overkill */
	{
	   /* was 1 */
	    m[i] +=
	       ( (x[j] + 0.01) *
	       zxm[bin[0]][bin[1]][bin[2]][bin[3]][bin[4]][bin[5]][bin[6]][j][i]
	       * 1.0 );
 	}
       }
    }
  }
  else /*RBF*/
     RbfInvMap(x,AMpar,m,P);



 }


/*
 * Use m[] cells to compute next position of the articulators and set AMpar[].
 *
 * The rate variable encodes the GO signal.  This function essentially 
 * computes the APV (Articulatory Position Vector) from the AVV (Articulatory
 * Velocity Vector or m[]).
 */
void apply_methods(void)
{
    int i;
    double dummy, dummy2;

#ifdef COMMENT
    for (i = 1; i <= NUM_ARTICULATORS; i++)
      {
	 OLD_ARTIC_POS[i][0] = ARTIC_POS[i][0];
	 OLD_ARTIC_POS[i][1] = ARTIC_POS[i][1];
      }
#endif COMMENT

   /* ensure that same rate is applied to all articulators */
   rate = get_rate();

   /**** Increase jaw height ****/
    jaw_height = AMpar[0];
    jaw_height = jaw_height + ((JAW_HEIGHT_FACTOR * rate * m[1]));
    if (jaw_height > MAX_JAW_HEIGHT) jaw_height = MAX_JAW_HEIGHT;

   /**** Decrease jaw height ****/
    jaw_height = jaw_height - ((JAW_HEIGHT_FACTOR * rate * m[2]));
    if (jaw_height < MIN_JAW_HEIGHT) jaw_height = MIN_JAW_HEIGHT;

    if (BLOCKED[0] == 0) AMpar[0] = jaw_height;


   /**** Increase tongue body position ****/
    tongue_body_position = AMpar[1];
    tongue_body_position = tongue_body_position +
      ((TONGUE_BODY_POSITION_FACTOR * rate * m[3]));
    if (tongue_body_position > MAX_TONGUE_BODY_POSITION)
      tongue_body_position = MAX_TONGUE_BODY_POSITION;

   /**** Decrease tongue body position ****/
    tongue_body_position = tongue_body_position -
      ((TONGUE_BODY_POSITION_FACTOR * rate * m[4]));
    if (tongue_body_position < MIN_TONGUE_BODY_POSITION)
      tongue_body_position = MIN_TONGUE_BODY_POSITION;

    if (BLOCKED[1] == 0) AMpar[1] = tongue_body_position;


   /**** Increase tongue body shape ****/
    tongue_body_shape = AMpar[2];
    tongue_body_shape = tongue_body_shape + 
      ((TONGUE_BODY_SHAPE_FACTOR * rate * m[5]));
    if (tongue_body_shape > MAX_TONGUE_BODY_SHAPE) 
      tongue_body_shape = MAX_TONGUE_BODY_SHAPE;

   /**** Decrease tongue body shape ****/
    tongue_body_shape = tongue_body_shape -
      ((TONGUE_BODY_SHAPE_FACTOR * rate * m[6]));
    if (tongue_body_shape < MIN_TONGUE_BODY_SHAPE) 
      tongue_body_shape = MIN_TONGUE_BODY_SHAPE;

    if (BLOCKED[2] == 0) AMpar[2] = tongue_body_shape;
 

    /**** Increase tongue tip position ****/
    tongue_tip_position = AMpar[3];
    tongue_tip_position = tongue_tip_position +
      ((TONGUE_TIP_POSITION_FACTOR * rate * m[7]));
    if (tongue_tip_position > MAX_TONGUE_TIP_POSITION)
      tongue_tip_position = MAX_TONGUE_TIP_POSITION;

    /**** Decrease tongue tip position ****/
    tongue_tip_position = tongue_tip_position -
      ((TONGUE_TIP_POSITION_FACTOR * rate * m[8]));
    if (tongue_tip_position < MIN_TONGUE_TIP_POSITION)
      tongue_tip_position = MIN_TONGUE_TIP_POSITION;

    if (BLOCKED[3] == 0) AMpar[3] = tongue_tip_position;


   /**** Increase lip aperture ****/
    lip_aperture = AMpar[4];
    lip_aperture = lip_aperture +
      ((LIP_APERTURE_FACTOR * rate * m[9]));
    if (lip_aperture > MAX_LIP_APERTURE) lip_aperture = MAX_LIP_APERTURE;

   /**** Decrease lip aperture ****/
    lip_aperture = lip_aperture -
      ((LIP_APERTURE_FACTOR * rate * m[10]));
    if (lip_aperture < MIN_LIP_APERTURE) lip_aperture = MIN_LIP_APERTURE;

    
    if (BLOCKED[4] == 0) AMpar[4] = lip_aperture;


    /**** Increase lip protrusion ****/
    lip_protrusion = AMpar[5];
    lip_protrusion = lip_protrusion +
      ((LIP_PROTRUSION_FACTOR * rate * m[11]));
    if (lip_protrusion > MAX_LIP_PROTRUSION) 
      lip_protrusion = MAX_LIP_PROTRUSION;

    /**** Decrease lip protrusion ****/
    lip_protrusion = lip_protrusion -
      ((LIP_PROTRUSION_FACTOR * rate * m[12]));
    if (lip_protrusion < MIN_LIP_PROTRUSION)
      lip_protrusion = MIN_LIP_PROTRUSION;

    if (BLOCKED[5] == 0) AMpar[5] = lip_protrusion;


    /**** Increase larynx height ****/
    larynx_height = AMpar[6];
    larynx_height = larynx_height +
      ((LARYNX_HEIGHT_FACTOR * rate * m[13]));
    if (larynx_height > MAX_LARYNX_HEIGHT) 
      larynx_height = MAX_LARYNX_HEIGHT;

    /**** Decrease larynx height ****/
    larynx_height = larynx_height -
      ((LARYNX_HEIGHT_FACTOR * rate * m[14]));
    if (larynx_height < MIN_LARYNX_HEIGHT)
      larynx_height = MIN_LARYNX_HEIGHT;

    if (BLOCKED[6] == 0) AMpar[6] = larynx_height;


#ifdef COMMENT
   /**** Raise velum ****/
    velum_height = velum_height + 
      ((VELUM_HEIGHT_FACTOR * rate * m[15]));
    if (velum_height > MAX_VELUM_HEIGHT) 
      velum_height = MAX_VELUM_HEIGHT;

   /**** Lower velum ****/
    velum_height = velum_height -
      ((VELUM_HEIGHT_FACTOR * rate * m[16]));
    if (velum_height < MIN_VELUM_HEIGHT) 
      velum_height = MIN_VELUM_HEIGHT;
    
    if (velum_height < 0)
      x_velum_height = 0.0;
    else
      x_velum_height = velum_height;
#endif COMMENT


#ifdef COMMENT
   /**** Raise tongue tip ****/
    tongue_tip_height = tongue_tip_height + 
      ((TONGUE_TIP_HEIGHT_FACTOR * rate * m[5]));
    if (tongue_tip_height > MAX_TONGUE_TIP_HEIGHT) 
      tongue_tip_height = MAX_TONGUE_TIP_HEIGHT;


   /**** Lower tongue tip ****/
    tongue_tip_height = tongue_tip_height -
      ((TONGUE_TIP_HEIGHT_FACTOR * rate * m[6]));
    if ((tongue_tip_height + tongue_body_height) < 0)
      tongue_tip_height = (- tongue_body_height);
#endif COMMENT


#ifdef COMMENT
   /**** Lower bottom lip ****/
    lower_lip_height = lower_lip_height -
      ((LIP_HEIGHT_FACTOR * rate * m[8]));
    if (lower_lip_height < MIN_LIP_HEIGHT) lower_lip_height = MIN_LIP_HEIGHT;


   /**** Raise top lip ****/
    upper_lip_height = upper_lip_height -
      ((LIP_HEIGHT_FACTOR * rate * m[9]));
    if (upper_lip_height < MIN_LIP_HEIGHT) upper_lip_height = MIN_LIP_HEIGHT;


   /**** Lower top lip and raise bottom lip ****/
    dummy = 400 + jaw_height - 
      (lower_lip_height + ((LIP_HEIGHT_FACTOR * rate * m[7])));
    dummy2 = 300 + upper_lip_height +
      ((LIP_HEIGHT_FACTOR * rate * m[10]));
    if ((dummy2 - dummy) < 40)
      {
	  upper_lip_height = upper_lip_height +
	    ((LIP_HEIGHT_FACTOR * rate * m[10]));
	  if (upper_lip_height > MAX_LIP_HEIGHT) 
	    upper_lip_height = MAX_LIP_HEIGHT;
	  lower_lip_height = lower_lip_height + 
	    ((LIP_HEIGHT_FACTOR * rate * m[7]));
	  if (lower_lip_height > MAX_LIP_HEIGHT) 
	    lower_lip_height = MAX_LIP_HEIGHT;
      }
    else  /* check if jaw has moved up; otherwise leave lips where they were */
      {
	 dummy = 400 + jaw_height - lower_lip_height;
	 dummy2 = 300 + upper_lip_height;
	 if ((dummy2 - dummy) > 40)
	   {
	      lower_lip_height = lower_lip_height - 
		((dummy2 - dummy - 39.5)/2.0);
	      upper_lip_height = upper_lip_height - 
		((dummy2 - dummy - 39.5)/2.0);
	   }
      }
    ARTIC_POS[4][1] = 400 + jaw_height - lower_lip_height;
    ARTIC_POS[5][1] = 300 + upper_lip_height;
    if ((perturb_lip == 'y') &&	(ARTIC_POS[4][1] < lip_perturb_height))
      {
	  lower_lip_height = lower_lip_height - 
	    (lip_perturb_height - ARTIC_POS[4][1]);
	  ARTIC_POS[4][1] = 400 + jaw_height - lower_lip_height;
      }
#endif COMMENT

#ifdef COMMENT
    /***** Take care of remaining boundary conditions *****/

    /***** Tongue tip maximal extension ****/
    ARTIC_POS[2][0] = 270 + tongue_body_extension;
    ARTIC_POS[3][0] = 320 + tongue_body_extension + 
      tongue_tip_extension;
    if (ARTIC_POS[3][0] > 450) 
      {
	  tongue_tip_extension = tongue_tip_extension -
	    (ARTIC_POS[3][0] - 450);
	  ARTIC_POS[3][0] = 450;
      }


    /**** Tongue body with top of mouth ****/
    ARTIC_POS[2][1]= 400 + ((double)(jaw_height) * 
       (((double)tongue_body_extension + 50.0) / 230.0)) - tongue_body_height;
    if (ARTIC_POS[2][1] < (301 - 20))
      {
	  tongue_body_height = tongue_body_height - 
	    ((301 - 20) - ARTIC_POS[2][1]);
	  ARTIC_POS[2][1] = (301 - 20);
      }
    if (ARTIC_POS[2][1] < 301)
      {
	  x_tongue_body_height = tongue_body_height - 
	    (301 - ARTIC_POS[2][1]);
	  x_tongue_body_y_location = 301;
      }
    else
      {
	  x_tongue_body_height = tongue_body_height;
	  x_tongue_body_y_location = ARTIC_POS[2][1];
      }


    /**** Tongue tip with jaw ****/
    if ((tongue_tip_height + x_tongue_body_height) < 0)
      {
	  tongue_tip_height  = (- x_tongue_body_height);
      }


    /**** Tongue tip with bottom tooth ****/
    if ((ARTIC_POS[3][0] > 420) & 
	(ARTIC_POS[3][1] > 
	 (399 + jaw_height - 
	  (50.0 * ((ARTIC_POS[3][0] - 420.0)/30.0)))))
      {
	  tongue_tip_height = tongue_tip_height - 
	    (399 + jaw_height - 
	     (50.0 * ((ARTIC_POS[3][0] - 420.0)/30.0)) -
	     ARTIC_POS[3][1]);
      }


    /**** Tongue tip with roof of mouth ****/
    ARTIC_POS[3][1] = x_tongue_body_y_location - tongue_tip_height;
    if (ARTIC_POS[3][1] < (301 - 20))
      {
	  tongue_tip_height = tongue_tip_height - 
	    ((301 - 20) - ARTIC_POS[3][1]);
	  ARTIC_POS[3][1] = (301 - 20);
      }
    if (ARTIC_POS[3][1] < 301)
      {
	  x_tongue_tip_height = tongue_tip_height - 
	    (301 - ARTIC_POS[3][1]);
	  x_tongue_tip_y_location = 301;
      }
    else
      {
	  x_tongue_tip_height = tongue_tip_height;
	  x_tongue_tip_y_location = ARTIC_POS[3][1];
      }


    /**** Tongue tip with top tooth ****/
    if ((ARTIC_POS[3][0] > 420) &&
	(ARTIC_POS[3][1] < 
	 ((302 - 20) + (50.0 * ((ARTIC_POS[3][0] - 420.0)/30.0)))))
      {
	  tongue_tip_height = tongue_tip_height - 
	    ((302 - 20) + (50.0 * ((ARTIC_POS[3][0] - 420.0)/30.0)) -
	     ARTIC_POS[3][1]);
	  ARTIC_POS[3][1] = x_tongue_body_y_location - tongue_tip_height;
      }
    if ((ARTIC_POS[3][0] > 420) & 
	(ARTIC_POS[3][1] < 
	 (302 + (50.0 * ((ARTIC_POS[3][0] - 420.0)/30.0)))))
      {
	  x_tongue_tip_height = tongue_tip_height - 
	    (302 + (50.0 * ((ARTIC_POS[3][0] - 420.0)/30.0)) -
	     ARTIC_POS[3][1]);
	  x_tongue_tip_y_location = x_tongue_body_y_location - 
	    x_tongue_tip_height;
	  tip_dental_stop = 1;
      }
    else
      tip_dental_stop = 0;


    /**** lip collision ****/
#ifdef COMMENT
    ARTIC_POS[4][1] = 400 + jaw_height - lower_lip_height;
    ARTIC_POS[5][1] = 300 + upper_lip_height;
    if ((ARTIC_POS[4][1] + 40) < ARTIC_POS[5][1])
      {
	  dummy = ARTIC_POS[5][1] - (ARTIC_POS[4][1] + 40);
	  lower_lip_height = lower_lip_height - 
	    (dummy * ((m[7] + 0.00005) / (m[7] + m[10] + 0.0001)));
	  upper_lip_height = upper_lip_height - 
	    (dummy * ((m[10] + 0.00005) / (m[7] + m[10] + 0.0001)));
	  if (lower_lip_height < MIN_LIP_HEIGHT)
	    {
		upper_lip_height = upper_lip_height - 
		  (MIN_LIP_HEIGHT - lower_lip_height);
		lower_lip_height = MIN_LIP_HEIGHT;
	    }
	  if (upper_lip_height < MIN_LIP_HEIGHT)
	    {
		lower_lip_height = lower_lip_height - 
		  (MIN_LIP_HEIGHT - upper_lip_height);
		upper_lip_height = MIN_LIP_HEIGHT;
	    }
	  ARTIC_POS[4][1] = 400 + jaw_height - 
	    lower_lip_height;
	  ARTIC_POS[5][1] = 300 + upper_lip_height;
      }
#endif COMMENT


    if (ARTIC_POS[4][1] < ARTIC_POS[5][1])
      {
	  x_lower_lip_height = lower_lip_height - 
	    ((ARTIC_POS[5][1] - ARTIC_POS[4][1])/2.0);
	  x_lower_lip_location = 400 + jaw_height - 
	    x_lower_lip_height;
	  x_upper_lip_height = upper_lip_height - 
	    ((ARTIC_POS[5][1] - ARTIC_POS[4][1])/2.0);
	  x_upper_lip_location = 300 + x_upper_lip_height;
      }
    else
      {
	  x_lower_lip_height = lower_lip_height;
	  x_lower_lip_location = ARTIC_POS[4][1];
	  x_upper_lip_height = upper_lip_height;
	  x_upper_lip_location = ARTIC_POS[5][1];
      }

    /*** set remaining articulator positions ***/
    ARTIC_POS[1][0] = 450;
    ARTIC_POS[1][1] = 350 + jaw_height;
    ARTIC_POS[4][0] = 450 + lip_protrusion;
    ARTIC_POS[5][0] = 450 + lip_protrusion;
    ARTIC_POS[6][0] = 220 + ((x_velum_height / MAX_VELUM_HEIGHT) * 14.0);
    ARTIC_POS[6][1] = 300 + ((x_velum_height / MAX_VELUM_HEIGHT) * 20.0);;
#endif COMMENT
}


/* 
 * Computes area function, min area, and formants and
 * sets global variable valid_configuration.
 *
 * Precondion:AMpar must be set before calling this function, because
 *  find_area_function() requires it.
 *
 * This function returns valid_configuration.
 */
int check_valid_configuration( void )
{
        int i;
	double min_area = 1000.0;

	valid_configuration=1;

        for(i=0;i<7;i++)
           if(fabs(AMpar[i]) > 3.0) return(0);
	find_area_function();

	if ((min_area=get_min_area()) < 0.01) {
		valid_configuration=0;
                /*printf("min area = %.2lf\n",min_area);*/
/*		if (!babbling_flag)
	       printf("invalid configuration: min_area = %f\n", min_area);*/
	}

	/* must always calculate formants when "speaking" */
	if (valid_configuration || !babbling_flag){
		if (get_formants() == 0){
			valid_configuration=0;
		     }
               /*  printf("%.2lf %.2lf\n",syn_targets[0],syn_targets[1]);*/
	     }
	return(valid_configuration);
}


double get_min_area( void )
{
	double max_area = 0.0;
	double min_area = 1000.0;
	int i;

	max_area = 0.0;
	min_area = 1000.0;
	for(i=0; i<nss; i++) {
		if (afvt[i].A < min_area)
			min_area = afvt[i].A;
#ifdef COMMENT
		else
			if (afvt[i].A > max_area)
				max_area = afvt[i].A;
#endif COMMENT
	}
	return(min_area);
}


/*
 * This function brings together the various ways of calculating the
 * formants, given the maeda parameters.
 * 
 * cal_tf_FA(), which is provided by the maeda software, is the slowest,
 * but potentially the most accurate.  However, I have it set up to return
 * frequencies that are no more accurate than +/- 10 Hz in order to 
 * increase its speed.
 *
 * rbf(), provided by Seth Cameron, is the radial basis function
 * approximation of the mapping from maeda parameters to formant
 * frequencies.  It is about 5 times faster than cal_tf_FA(), but
 * not very accurate around the edges.  Also, it gives screwy results
 * sometimes.
 *
 * interpolate_dict(), provided by Dave Johnson, uses a dictionary which
 * is constructed using cal_tf_FA() for 7 of the parameters.  Because it
 * interpolates the values, its behavior is smoother.  It is also very
 * accurate over most of the range of values.  Also, it is about 20 times
 * faster than rbf().
 *
 * This function assumes that AMpar[] contains the maeda parameters.  It
 * writes the formant frequencies into syn_targets[].
 *
 * This function returns 1 if I believe that the three returned formants are
 * accurate, and 0 otherwise.
 *
 * This function attempts to return the most accurate value of the formants
 * over the widest range of maeda parameters as fast as possible.
 */




int get_formants( void )
{
	int nfrmmax = 3;     /* number of formant frequencies desired */
	int num_formants;
	double amp[8],temptargets[8];
	int i;
 
	if (which_fmap==0 && interpolate_dict(AMpar, syn_targets) == 1 ){
	  if(NUM_TARGETS > 3){ 
                get_magnitudes();
                syn_targets[3] = AV;
                if(syn_targets[3] > 600){ 
                    printf("!!!!!!!!!too big mag\n");
                    getchar();
                 }
          }
          if(FIX_JAW)  adjust_formants();
           return(1);
	}
	else
	  {
           if(which_fmap==1)  /* using rbf forward map */
	     {

               RbfForMap(AMpar, syn_targets, P);

               if(NUM_TARGETS > 3){
                     if(syn_targets[3] > 600.0) syn_targets[3] = 600.0;
                     if(syn_targets[3] < 0.0) syn_targets[3] = 0.0;
	       }
               if((syn_targets[2] < syn_targets[1])||
                  (syn_targets[1] < syn_targets[0]))
		 {
 
                  printf("r %lf %lf %lf\n",syn_targets[0],syn_targets[1],
                     syn_targets[2]);
                  find_area_function();
	          cal_tf_FA(nfrmmax, syn_targets, amp, &num_formants);
                  printf("m %lf %lf %lf \n",syn_targets[0],syn_targets[1],
                     syn_targets[2]);
                  if(NUM_TARGETS > 3) printf("mag: %lf\n",syn_targets[3]);
                  printf("\n");
		  /* getchar();*/
		 }
	 if(NUM_TARGETS > 3)
          if(syn_targets[3] > 600){ 
                 printf("!!!!!!!!!too big\n");
                 getchar();
          }
          if(FIX_JAW) adjust_formants();
            return(1);
	    }
           else 
	   {
		/* use maeda calculation of formant frequencies */

                find_area_function();
		cal_tf_FA(nfrmmax, syn_targets, amp, &num_formants);
  

		if(NUM_TARGETS > 3){
                    get_magnitudes();
                    syn_targets[3] = AV;
                }



/*        		cal_tf_FA(6, temptargets, amp, &num_formants);
	
if (FIX_JAW == 1)

               {
                printf("No. of formants:  %d\n",num_formants);
                for (i = 0; i < num_formants; i++)
                  printf("F%1d = %.1lf ",(i+1),temptargets[i]);
                printf("\n");
                for (i = 0; i < num_formants; i++)
                  printf("AmpF%1d = %.1lf ", (i+1),amp[i]);
                printf("\n");
	     }
*/

		/* printf("using cal_tf_FA()\n"); */
           if(NUM_TARGETS > 3)
              if(syn_targets[3] > 600){ 
                  printf("!!!!!!!!!too big mag\n");
                  getchar();
               }
           if(FIX_JAW) adjust_formants();

		if (num_formants < 3)
			return(0);
		else
			return(1);
	     }
	}

#ifdef COMMENT
	rbf(AMpar, syn_targets);
#endif COMMENT
}


/*
  Preconditions:
     AMpar must be set before calling this function because find_area_function()
uses it. 
*/

int get_magnitudes(void)
{
  double min_area,start_fric,stop_voice=0.2;

             start_fric = 0.3;

             find_area_function();
             min_area = get_min_area();

#ifdef COMMENT
             inc_size = 0.1;
             /* Control frication */
             if ((min_area < start_fric) && (inc_size < 0))
               {
                  if (min_area > (start_fric/2.0))
                    AF = 600.0 * (1.0 - ((min_area - (start_fric/2.0))
                                      /(start_fric/2.0)));
                  else
                    AF = 600.0 * (1.0 - (((start_fric/2.0) - min_area)
                                      /(start_fric/2.0)));
                  /*  printf("AF = %lf\n",AF); */
               }
             else
#endif COMMENT
               AF = 0;

             /* Control voicing */
                  if ((min_area-0.05) < stop_voice)
                    {
                       AV = 600. * ((min_area - 0.05)/stop_voice);
                       if (AV < 0.0) AV = 0.0;
                    }
                  else
                    AV = 600.;

return(1);
}

/* Adaptation of Dave's get_methods to use as test routine */

void Mget_methods(double *dformants,double *darts)
{
    int i,j;

   
     get_tessellated_bin();

#ifdef COMMENT
    if (!babbling_flag) print_bin();
#endif COMMENT

     for (i = 1; i <= 7; i++) {
       darts[i-1] = 0.0;
       for (j = 1; j <= 2; j++) {

	 if ( (dformants[j-1] *
           zxm[bin[0]][bin[1]][bin[2]][bin[3]][bin[4]][bin[5]][bin[6]][j][i]) >
		0.0  &&  dformants[j-1] > 0.0001) /* overkill */
	{
	   /* was 1 */
	    darts[i-1] +=
	       ( (dformants[j-1] + 0.01) *
	       zxm[bin[0]][bin[1]][bin[2]][bin[3]][bin[4]][bin[5]][bin[6]][j][i]
	       * 1.0 );
 	}
       }
    }

 }


void adjust_formants()
{
    syn_targets[0] -= 400;
    syn_targets[1] -= 100;
    syn_targets[2] -= 700;
}
