#include "diva.h"
#include "babble.h"
#include "../rbf_src/rbf.h"
#include "../maeda_code/plot_lib.h"
#include "diva_dir.h"

#include <X11/Intrinsic.h> 
#include <X11/StringDefs.h>
#include <Xm/Xm.h>
#include <Xm/XmStrDefs.h>
#include <Xm/Text.h>

#include "global_vars.h"


void delay(void); 
void rest_config(void);
void get_state(void);
void normalize_formants(double *,double *);
int normalize_ratios(double *,double *);
int get_tessellated_bin(void);
void get_x(void);
void learn_methods(void);
void learn_phonemes(void);
double dzxm_dt(int *, double, double);
double dzpx_dt(int *, double, double);
double rectify(double);
void init_ideal_targets(void);
void init_weights(void);
void init_sensyn( void );
void print_weights(FILE *);
void load_weights(FILE *);
void load_rbf_weights(FILE *);
double INTEGRATOR(double (),int *,double,double,double);
void talker(void);
void babble(Widget w, caddr_t data, XEvent *p_event);
void offline_babbling( void );
int get_phoneme_index(char);
void get_command_hypercube(double *, int);
void get_feedback_hypercube(double *, int);
void print_zxm( void );
void print_frm( void );
double max(double, double);
double min(double, double);
void region_velocity_babbling( void );
void random_velocity_babbling( void );
void random_phoneme_babbling( void );
void wait_for_continue( void );
void wait_for_key_stroke( void );
extern void flush_sound( void );
double get_rate( void );
int check_valid_configuration( void );
double get_min_area( void );
void print_bin( void );
double get_etaB();
double get_alpha();



double XSUM_THRESHOLD = 0.1;
double dorsum_constrict_max=6.0;
double max_aperture=10.1;

int jaw_phase = 0;
/* typical set of max's and min's produced by rbf */
double f1_min=50.0 /*f1_min = 10.0*/, f1_max=1200.0;
double f2_min=500.0, f2_max=3500.0;
double f3_min=1600.0, f3_max=4000.0;

int bin[7];
int old_bin[7];
int bin_count[NT0][NT1][NT2][NT3][NT4][NT5][NT6];
int coverage=0;		/* 1 ==> every bin sampled uniformly */
int babbling_flag=0;
int offline_babbling_flag=0;
int erase_flag=1;

int NT[7] = {NT0,NT1,NT2,NT3,NT4,NT5,NT6};
double max_artic[7] = {
			MAX_JAW_HEIGHT,
			MAX_TONGUE_BODY_POSITION,
			MAX_TONGUE_BODY_SHAPE,
			MAX_TONGUE_TIP_POSITION,
			MAX_LIP_APERTURE,
			MAX_LIP_PROTRUSION,
			MAX_LARYNX_HEIGHT
			};

double min_artic[7] = {
			MIN_JAW_HEIGHT,
			MIN_TONGUE_BODY_POSITION,
			MIN_TONGUE_BODY_SHAPE,
			MIN_TONGUE_TIP_POSITION,
			MIN_LIP_APERTURE,
			MIN_LIP_PROTRUSION,
			MIN_LARYNX_HEIGHT
			};

extern area_function *afvt;
extern int nss, nph, nbu;
extern valid_configuration;
extern double rate;
extern double drand48();
extern double get_SF0();
extern void srand48(long seedval);
/**************************************************************/
/**********************  Learn phonemes ***********************/
/**************************************************************/


/*
WARNING:  
 The fscanfs into instring are a potential memleak (if it tries 
to read some long string of garbage in and writes past the end of instring)
*/

void init_diva(void)
{
    extern void read_parameter_file();
    char filename[] = PARAMS;
    char instring[100];
    FILE *infile;
    int i;

    load_dict();

    if ((infile = fopen(filename,"r")) != NULL)
      fscanf(infile,"%s",instring);
    else
      {
	  printf("File 'parameters.dat' does not exist --\n");
	  printf("cannot use 'Change Parameters' option.\n");
      }
    if (strcmp(instring,"%%ParameterFile") == 0)
	  read_parameter_file(infile);
    else
      {
	  printf("File 'parameters.dat' is not a parameter file --\n");
	  printf("cannot use 'Change Parameters' option.\n");
      }


    strcpy(filename,WEIGHTS);
    if ((infile = fopen(filename,"r")) != NULL)
      fscanf(infile,"%s",instring);
    else
      {
	  printf("File 'weights.dat' does not exist --\n");
      }
    if (strcmp(instring,"%%WeightFile") == 0)
	  load_weights(infile);
    else
      {
	  printf("File 'weights.dat' is not a weight file --\n");
      }


    /*
     * This is necessary because read_parameter_file() sets
     * babbling_graphics_on and it needs to be 0.0 for the rest_config()
     * call to work when babbling offline.
     */
    if (offline_babbling_flag) babbling_graphics_on = 0.0;


    init_ideal_targets();   /* initializes ideal formant and ratio targets */
    rest_config();
    init_maeda();
    init_sensyn();

    if(RBF==1)
      {
         P = CreateRbf();

#ifdef COMMENT
        strcpy(filename,RBF_WEIGHTS);
        if ((infile = fopen(filename,"r")) != NULL)
          fscanf(infile,"%s",instring);
        else
        {
   	  printf("File 'rbf_weights.dat' does not exist --\n");
         }
        if (strcmp(instring,"%%RbfWeightFile") == 0)
 	  load_rbf_weights(infile);  /* accesses rbf struct P */
        else
         { 
	  printf("File 'rbf_weights.dat' is not a weight file --\n");
          }
#endif COMMENT
      }

    rest_config();

    for (i=0; i<NUM_TARGETS; i++)
	syn_targets[i] = 0.0;
}


void babble(Widget w, caddr_t data, XEvent *p_event)
{
	extern Widget speak_button, babble_button;

	XtSetSensitive(play_demo_button, FALSE);                     
	XtSetSensitive(make_demo_button, FALSE);
	XtSetSensitive(plot_conf_button, FALSE);
	XtSetSensitive(fix_jaw_button, FALSE);
	XtSetSensitive(speak_button, FALSE);
	XtSetSensitive(babble_button, FALSE);
	XtSetSensitive(formant_button, FALSE);
	XtSetSensitive(ratio_button, FALSE);
	XtSetSensitive(load_weights_button, FALSE);
	XtSetSensitive(load_parameters_button, FALSE);
	XtSetSensitive(cap_button, FALSE);
	XtSetSensitive(synthesize_button, FALSE);

	babbling_flag=1;

	offline_babbling();

	XtSetSensitive(plot_conf_button, TRUE);
	XtSetSensitive(fix_jaw_button, TRUE);
	XtSetSensitive(speak_button, TRUE);
	XtSetSensitive(babble_button, TRUE);
	XtSetSensitive(formant_button, TRUE);
	XtSetSensitive(ratio_button, TRUE);
	XtSetSensitive(load_weights_button, TRUE);
	XtSetSensitive(load_parameters_button, TRUE);
	XtSetSensitive(cap_button, TRUE);
	XtSetSensitive(play_demo_button, TRUE);
	XtSetSensitive(synthesize_button, TRUE);
	XtSetSensitive(make_demo_button, TRUE);

	babbling_flag=0;
	update_maeda_sliders();
}


/*
 * This function is introduced only to allow offline_babbling without
 * graphics or other human interaction.
 */
void offline_babbling( void )
{
  /* RBF */ 
  char instring[100];

  extern void get_babble_methods();
  int i, j, k;
  int i0, i1, i2, i3, i4, i5, i6;
  FILE *fp_new_weights;
  time_t tloc ;
  long seedval;
  FILE  *wts;
 
  /*
   * After babbling commences, we do not allow the user to set the rate.
   */
  rate = (double)SPEAKING_RATE/100.0;

  FIX_JAW = 0;
  BREAK = 0;

  name_phonemes();
  init_weights();

  voicing = 1;
  noise = 0;
  if (babbling_graphics_on > 0.5)
	draw_voicing();

  for (i = 1; i <= NUM_PHONEMES; i++)
    {
      p[i] = 0;
      p_count[i] = 0;
      p_voice[i] = 0; 
      p_noise[i] = 0;
      p_velum[i] = 0;
      p_pressure[i] = 0;
    }

  rest_config();
  get_formants();
  get_state();
  get_x();

  if (!offline_babbling_flag) {
	shift_eye();
	flap_arms();
  }

  if(RBF==1)
    {
    /* Train forward & Inv maps */

     TrainRbf(*P);
  }
  else{

    /*
     * If coverage == 1, then each tessellated bin is sampled a number
     * times determined by the num_velocity_trials and the number of bin.
     * This allows more uniform coverage of the tessellated bins.
     * If coverage == 0, the articulator configurations are chosen randomly
     * as before.
     */
     /* effective rate initially small at this point */

     if (coverage == 1) {
	  rate = rate/2;
	  region_velocity_babbling(); /* region babbling requires small rate */
	  rate = rate*2;
     }
     else {
	  random_velocity_babbling(); /* random babbling requires large rate */
       }
  }

  printf("Beginning phoneme babbling... \n");
  
  /* flush the sound buffer (if necessary) */
  if (babbling_graphics_on > 0.5)
    flush_sound();
  
  rate = 5.0*rate;
  
  jaw_phase = 0;
  
  BREAK = 0;
  random_phoneme_babbling();/* random babbling requires large rate */
  
  /* flush the sound buffer (if necessary) */
  if (babbling_graphics_on > 0.5)
    flush_sound();
  
  for (k = 1; k <= NUM_PHONEMES; k++)
    {
       printf("%d %c 's were encountered.\n",p_count[k],p_name[k]);
    }
  for (k = 1; k <= NUM_PHONEMES; k++)
    {
       for (j = 1; j <= NUM_STATE_VARS; j+=2)
	 {
	    if ((zpx[k][j] + zpx[k][j+1]) > 1.001)
	      {
		 printf("Invalid hypercube learned for phoneme '%c'.\n",
			p_name[k]); 
		 j = NUM_STATE_VARS + 1;
	      }
	 }
    }
  
  voicing = 0;
  noise = 0;
  
  BREAK = 0;

  /* unconditionally save weights in the file 'new_weights.dat' */

  fp_new_weights = fopen("new_weights.dat", "w");
  print_weights(fp_new_weights);
  printf("Weights stored in '%s'\n", "new_weights.dat");
  fclose(fp_new_weights);

  if (!offline_babbling_flag) {
	shift_eye();
	flap_arms();
  }
}

/**************************************************************/
/***************** End phoneme learning ***********************/
/**************************************************************/



/**************************************************************/
/****************** Read out phonemes   ***********************/
/**************************************************************/

void speak(Widget w, caddr_t data, XEvent *p_event)
{
   double Error,bad[7];

   FILE *JInv,*Interp;
   double midpt[7];
   char tempstring[20];
   int i,j,k,index;
   int my_index,count,hypercube_reached;
   double xsum, h[NUM_STATE_VARS + 1], next_h[NUM_STATE_VARS + 1];
   double tonic = 0, temp;
   int overlap[NUM_STATE_VARS + 1];
   char dummy;
   char outname[50];
   double min_area;
   static FILE *fp_formants=NULL, *fp_artics=NULL;
   double initial_distance, current_distance;

   babbling_flag = 0;
   BREAK = 0;
   CONTINUE_AFTER_PHONEME = 0;
   XtSetSensitive(plot_conf_button, FALSE);
   XtSetSensitive(play_demo_button, FALSE);
   XtSetSensitive(make_demo_button, FALSE);
   XtSetSensitive(speak_button, FALSE);
   XtSetSensitive(formant_button, FALSE);
   XtSetSensitive(ratio_button, FALSE);
   XtSetSensitive(babble_button, FALSE);
   XtSetSensitive(load_weights_button, FALSE);
   XtSetSensitive(load_parameters_button, FALSE);
   XtSetSensitive(synthesize_button, FALSE);

   name_phonemes();

#ifdef COMMENT
  Error = ComputeError(P,bad);

  printf(" Worst Error is %lf at",Error);
  for(i=0;i<7;i++){
     printf(" %.4lf",bad[i]);
  }
  printf("\n");



   JInv = fopen("JInv","w");
   for(j=0;j<10;j++){
      fprintf(JInv,"%d: ",j);
      PrintJInv(P,((P->mu) + j*7),JInv);
   }
   fclose(JInv);
   Interp = fopen("Interp","w");
   fprintf(Interp,"Halfway\n");
   for(j=0;j<9;j++){
      fprintf(Interp,"%d-%d: ",j,j+1);
      for(i=0;i<7;i++){
         midpt[i] = (P->mu[j*7 +i] + P->mu[(j+1)*7 + i])/2.0;
         fprintf(Interp,"%.2lf ",midpt[i]);
      }
      fprintf(Interp,"\n\n");
      PrintJInv(P,midpt,Interp);
   }
   fprintf(Interp,"\n\nThirdway\n");
   for(j=0;j<9;j++){
      fprintf(Interp,"%d to %d: ",j,j+1);
      for(i=0;i<7;i++){
         midpt[i] = (P->mu[j*7 +i] + 2.0*P->mu[(j+1)*7 + i])/3.0;
         fprintf(Interp,"%.2lf ",midpt[i]);
      }
      fprintf(Interp,"\n\n");
      PrintJInv(P,midpt,Interp);
   }
   fclose(Interp);
 
   exit(0);
#endif COMMENT

   rate = get_rate();
   /*   get_formants(); here*/
   get_state();
   get_x();

   for (i = 1; ((i <= NUM_PHONEMES) && (BREAK != 1)); i++)
     p[i] = 0;

   voicing = 1;
   noise = 0;
   draw_voicing();
 
   /* here*/
   get_state();
   get_x();

   /* loop through all phonemes in specified phoneme string */
   if ((FORMANT_TARGET_FLAG == 1) || (RATIO_TARGET_FLAG == 1))
     {
	strcpy(tempstring,PHONEME_STRING);
	strcpy(PHONEME_STRING,"a"); /* just to make sure only one 
				       trip through loop is taken */
     }
   for (i = 0; ((i < strlen(PHONEME_STRING)) && (BREAK != 1)); i++)
     {
       /*        get_formants(); here*/
	get_state();
	get_x();

       my_index = get_phoneme_index(PHONEME_STRING[i]);
       current_distance = 0;
       initial_distance = 0;

	if ((FORMANT_TARGET_FLAG == 0) && (RATIO_TARGET_FLAG == 0)
	    && (my_index == (NUM_PHONEMES + 1)))
	  {
	     printf("Don't know phoneme '%c'\n",PHONEME_STRING[i]);
	  }
	else
	  {
	   if ((FORMANT_TARGET_FLAG == 0) && (RATIO_TARGET_FLAG == 0))
	     {
		printf("\nPronouncing '%c' ...",p_name[my_index]);
	     }
	   if (FORMANT_TARGET_FLAG == 1)
	     {
		printf("\nUsing formant target:  ");
		for (i = 0; (i < (NUM_STATE_VARS/2))&&(i<3); i++)
		  printf("F%1d = %.1lf ",(i+1),FORMANT_TARGET[i]);
		printf("\n");
	     }
	   if (RATIO_TARGET_FLAG == 1)
	     {
		printf("\nUsing ratio target:  ");
		for (i = 0; i < ((NUM_STATE_VARS/2)&&(i<3)); i++)
		  printf("R%1d = %.3lf ",(i+1),RATIO_TARGET[i]);
		printf("\n");
	     }
	   fflush(stdout);

	   if ((LIP_PERT == 1) && (p_name[my_index] == 'p'))
	     {
	       draw_lip_pert();
	       perturb_lip = 'y';
	     }
	   else
	     {
		 erase_lip_pert();
		 perturb_lip = 'n';
	     }
	   
	   if ((JAW_PERT == 1) && ((p_name[my_index] == 'b') ||
				   (p_name[my_index] == 'p') ||
				   (p_name[my_index] == 'z')))
	     {
	       draw_jaw_pert();
	       perturb_jaw = 'y';
	     }
	   else
	     {
		 erase_jaw_pert();
		 perturb_jaw = 'n';
	     }
	   
	   p_count[my_index] = 0;

	   count = 0;

	   do
	     {
	       /*  Check for break, speed change, etc. happens in delay() */
	       delay();

	       tonic = (1-get_rate()) * min_cube_factor + 0.01;

	       if (BREAK != 0) 
		 {
		    printf("Break Encountered.\n");
		 }
	       
	       for (j = 1; j <= NUM_PHONEMES; j++)
		 p[j] = 0;
	       p[my_index] = 1; /* this is "reset" by get_phonemes */

	       /*     get_formants(); here*/
	       get_state();

	       if ((FORMANT_TARGET_FLAG == 0) && (RATIO_TARGET_FLAG == 0))
		 {
		    get_command_hypercube(h, 
					get_phoneme_index(PHONEME_STRING[i]));

		    /* implement anticipatory coarticulation */
		    for (k = 1; k <= NUM_STATE_VARS; k++)
		      overlap[k] = 1;
		    for (j = (i + 1); 
			 ((j < strlen(PHONEME_STRING)) && 
			  (j < (i + 1 + (int)(coartic_distance)))); j++)
		      {
			 get_command_hypercube(next_h, 
				      get_phoneme_index(PHONEME_STRING[j]));
			 
			 for (k = 1; k < NUM_STATE_VARS; k+=2)
			   {
			      if ((next_h[k] > h[k]) &&
				  (next_h[k] < (1 - h[k+1])) &&
				  (overlap[k] == 1))
				h[k] = next_h[k];
			      
			      if ((next_h[k+1] > h[k+1]) &&
				  (next_h[k+1] < (1 - h[k])) &&
				  (overlap[k] == 1))
				h[k+1] = next_h[k+1];
			      
			      if ((next_h[k] > (1 - h[k+1])) ||
				  (next_h[k+1] > (1 - h[k])))
				{
				   overlap[k] = 0;
				 overlap[k+1] = 0;
				}
			   }
		      }

		    current_distance = 0;
		    for (j = 1; j <= NUM_STATE_VARS; j+=2)
		      {
			 x[j] = h[j] - s[j] + tonic;
			 x[j+1] = h[j+1] - s[j+1] + tonic;
		      }
		 }
	       else
		 {
		    get_x();
		 }

	       /* lateral inhibition between ODV ag/antag pair */
	       if ((x[j] > 0.0) && (x[j+1] > 0.0))
		 {
		    temp = x[j];
		    x[j] = x[j] - x[j+1];
		    x[j+1] = x[j+1] - temp;
		 }
	       
#ifdef COMMENT
	       /* rectify ODV */
	       if (x[j] < 0.0) x[j] = 0.0;
	       if (x[j+1] < 0.0) x[j+1] = 0.0;
	       
#endif COMMENT

	       if (BREAK != 0)
		 {
		    printf("Commanded DV's:\n");
		    for (j = 1; j <= NUM_STATE_VARS; j+=2)
		      {
			 printf("x[%d] = %lf, x[%d] = %lf, h[%d] = %lf, ",
				j,x[j],(j+1),x[j+1],j,h[j]);
			 printf("h[%d] = %lf\n",(j+1),h[j+1]);
		      }
		 }

	       get_methods();
	       
	       /* erase_articulators(); */

	       if ((JAW_PERT == 1) && ((p_name[my_index] == 'b') ||
				       (p_name[my_index] == 'p') ||
				       (p_name[my_index] == 'z')))
		 draw_jaw_pert();
	       if ((LIP_PERT == 1) && (p_name[my_index] == 'p'))
		 draw_lip_pert();

	       apply_methods();
	       check_valid_configuration();

#ifdef COMMENT
if (FIX_JAW)  /* Lip tube simulation */
{
   syn_targets[0] = syn_targets[0] + 0;  /*75*/


   syn_targets[1] = syn_targets[1] +200;/*340, 650 according to SP&0 estimate */

   if (NUM_TARGETS > 2)
     syn_targets[2] = syn_targets[2] + 0;
}	
#endif COMMENT
/*       
syn_targets[0] = syn_targets[0] - 50;
syn_targets[1] = syn_targets[1] - 580;
if (syn_targets[1] < syn_targets[0])
{
   temp = syn_targets[0];
   syn_targets[0] = syn_targets[1];
   syn_targets[1] = temp;
}
*/
#ifdef COMMENT
	       /* set noise or voicing correctly */
	       if (min_area=get_min_area() < stop_voice) {
			syn_targets[3] = 0.0;
			syn_targets[4] = 600.0;
	       }
	       else {
			syn_targets[3] = 600.0;
			syn_targets[4] = 0.0;
	       }
#endif COMMENT

	       if(!RBF) get_tessellated_bin();

	       voicing = p_voice[my_index];
	       velum = p_velum[my_index];
	       noise = p_noise[my_index];
	       pressure = p_pressure[my_index];
               update_maeda_sliders();
               display_formants();
               
              
	       get_phonemes();

	       /*
		* Compute area function, draw articulators, and compute
		* speech samples (if necessary).
		*/
	       process_sound_frame();

	    
	       frm_plot(syn_targets);

	       count = count + 1;

	       if ((FORMANT_TARGET_FLAG == 0) && (RATIO_TARGET_FLAG == 0))
		 {
		    get_feedback_hypercube(h, 
				       get_phoneme_index(PHONEME_STRING[i]));
		    hypercube_reached = 1;
		    for (j = 1; j <= NUM_STATE_VARS; j+=2)
		      {
			 x[j] = h[j] - s[j] + tonic;
			 x[j + 1] = h[j + 1] - s[j + 1] + tonic;
			 if ((x[j] > 0.0) && (x[j+1] > 0.0))
			   {
			      temp = x[j];
			      x[j] = x[j] - x[j+1];
			      x[j+1] = x[j+1] - temp;
			   }
			 /* rectify ODV */
			 if (x[j] < 0.0) x[j] = 0.0;
			 if (x[j+1] < 0.0) x[j+1] = 0.0;
			 
			 if ((x[j] > 0.01) || (x[j + 1] > 0.01))
			   hypercube_reached = 0;
		      }
		    
		    if (BREAK != 0)
		      {
			 printf("Feedback DV's:\n");
			 for (j = 1; j <= NUM_STATE_VARS; j+=2)
			   {
			      printf("x[%d] = %lf, x[%d] = %lf, h[%d] = %lf, ",
				     j,x[j],(j+1),x[j+1],j,h[j]);
			      printf("h[%d] = %lf\n",(j+1),h[j+1]);
			   }
		      }
		 }

	     } while (/* (hypercube_reached == 0) && (p[my_index] == 0) && */
		      (count < 200) && (BREAK != 1));
	     /* was count < 150 */

	   shift_eye();
	   flap_arms();
	   /* flush the sound buffer (if necessary) */
           flush_sound();
for(index=1;index<12;index++){
 if (p[index] == 1)
  printf(" vowel %c %d produced according to hearer.c;\n",p_name[index],index);
}
	   if ((FORMANT_TARGET_FLAG == 0) && (RATIO_TARGET_FLAG == 0))
	     {
		printf("   hypercube_reached = %d;\n", hypercube_reached);
		if (p[my_index] == 1)
		  printf("   vowel produced according to hearer.c;\n");
	     }
	   if (BREAK == 1)
	     printf("   aborted after %d iterations.\n", count);
	}
	if ((stop_after_phoneme > 0.1) && (i < (strlen(PHONEME_STRING) - 1)))
	  {
	     wait_for_continue();
	  }
     }
    
   if ((FORMANT_TARGET_FLAG == 1) || (RATIO_TARGET_FLAG == 1))
     {
	strcpy(PHONEME_STRING,tempstring);
     }
   
   voicing = 0;
   noise = 0;
   
   which_fmap  = 2.0;
   get_formants();
   printf("maeda at %lf %lf %lf\n", syn_targets[0],syn_targets[1],syn_targets[2]);
   which_fmap  = 0;
   get_formants();
   printf("dict at %lf %lf %lf\n", syn_targets[0],syn_targets[1],syn_targets[2]);


   which_fmap = 1.0;
   get_formants();

   printf("rbfs at %lf %lf %lf\n", syn_targets[0],syn_targets[1],syn_targets[2]);
   which_fmap = 1.0;
   shift_eye();
   flap_arms();
   perturb_jaw = 'n';
   perturb_lip = 'n';
   XtSetSensitive(plot_conf_button, TRUE);
   XtSetSensitive(play_demo_button, TRUE);
   XtSetSensitive(speak_button, TRUE);
   XtSetSensitive(formant_button, TRUE);
   XtSetSensitive(ratio_button, TRUE);
   XtSetSensitive(babble_button, TRUE);
   XtSetSensitive(load_weights_button, TRUE);
   XtSetSensitive(load_parameters_button, TRUE);
   XtSetSensitive(synthesize_button, TRUE);
   XtSetSensitive(make_demo_button, TRUE);

   BREAK = 0;
}

/**************************************************************/
/****************** End phoneme readout ***********************/
/**************************************************************/


void get_command_hypercube(double *h, int phoneme_index)
{
    int i;

    for (i = 1; i <= NUM_STATE_VARS; i += 2)
      {

/*	 
	  h[i] = zpx[phoneme_index][i];
	  h[i + 1] = zpx[phoneme_index][i + 1];

	  h[i] = zpx[phoneme_index][i] + 
	    ((1.0 - (zpx[phoneme_index][i] + zpx[phoneme_index][i + 1])) *
	     ((1.0 - ((double)SPEAKING_RATE / 100.0)) / 2.0));

	  h[i + 1] = zpx[phoneme_index][i + 1] +
	    ((1.0 - (zpx[phoneme_index][i] + zpx[phoneme_index][i + 1])) *
	     ((1.0 - ((double)SPEAKING_RATE / 100.0)) / 2.0));
*/		     

	 h[i] = zpx[phoneme_index][i]; 
/*
	 +
	    ((1.0 - (zpx[phoneme_index][i] + zpx[phoneme_index][i + 1])) *
	     0.5 * 
	     (1.0 - (min_cube_factor + (((double)SPEAKING_RATE / 125.0) *
					(1.0 - min_cube_factor)))));
		     
*/

	  h[i + 1] = zpx[phoneme_index][i + 1];
/*	 
	 +
	    ((1.0 - (zpx[phoneme_index][i] + zpx[phoneme_index][i + 1])) *
	     0.5 *
	     (1.0 - (min_cube_factor + (((double)SPEAKING_RATE / 125.0) *
					(1.0 - min_cube_factor)))));
     printf("%lf %lf ",h[i],h[i+1]);
*/


      }

}


void get_feedback_hypercube(double *h, int phoneme_index)
{
    int i;

    for (i = 1; i <= NUM_STATE_VARS; i += 2)
      {
/*
	  h[i] = zpx[phoneme_index][i] + 
	    ((1.0 - (zpx[phoneme_index][i] + zpx[phoneme_index][i + 1])) *
	     ((1.0 - ((double)SPEAKING_RATE / 100.0)) / 3.0));

	  h[i + 1] = zpx[phoneme_index][i + 1] +
	    ((1.0 - (zpx[phoneme_index][i] + zpx[phoneme_index][i + 1])) *
	     ((1.0 - ((double)SPEAKING_RATE / 100.0)) / 3.0));
*/
	 h[i] = zpx[phoneme_index][i];
/*
         + 
	    ((1.0 - (zpx[phoneme_index][i] + zpx[phoneme_index][i + 1])) *
	     0.5 * 
	     (1.0 - (min_cube_factor + (((double)SPEAKING_RATE / 105.0) *
					(1.0 - min_cube_factor)))));
*/		
     
	  h[i + 1] = zpx[phoneme_index][i + 1];

/* 
         +
	    ((1.0 - (zpx[phoneme_index][i] + zpx[phoneme_index][i + 1])) *
	     0.5 *
	     (1.0 - (min_cube_factor + (((double)SPEAKING_RATE / 105.0) *
					(1.0 - min_cube_factor)))));
*/
      }
}


void get_babble_methods(int jaw_phase)
{
    int j;

    if (strcmp(BABBLE_TYPE,"random") == 0)
      {
#ifdef COMMENTS
	  for (j = 1; j <= NUM_METHODS; j++)
	    if ((random() % 3) == 1) 
		m[j] = 1;
	    else
	        m[j] = 0;
#endif COMMENTS

	  /* This ensures that if an articulator is activated, only one
	   * of the agonist and antagonist will be activated.
	   */
	  for (j = 1; j <= NUM_METHODS; j += 2) {
	    m[j] = 0;
	    m[j+1] = 0;
	    /* decide if articulator should be activated */
	    if ((random() % 3) == 1) {
	       /* decide if agonist or antagonist should be activated */
	       if ((random() % 3) == 1) {
		   m[j] = 1;
	       }
	       else {
	           m[j+1] = 1;
	       }
	    }

	  }
      }
    if (strcmp(BABBLE_TYPE,"variegated") == 0) 
      {
	  fprintf(stderr, "variegated babbling not supported\n");
	  exit(1);

#ifdef COMMENT
	  if (jaw_phase == 0)
	    {
		m[2] = 0.7 + (0.3 * ((double)(random() % 1000)/1000.0));
		m[1] = 1.0 - m[2];
		
		for (j = 3; j <= NUM_METHODS; j++)
		  if ((random() % 3) == 1) 
		    m[j] = 1;
		  else
		    m[j] = 0;
	    }
	  else
	    {
		m[2] = m[1];
		m[1] = 1.0 - m[2];
		for (j = 3; j <= NUM_METHODS; j++)
		  m[j] = 0;
	    }
#endif COMMENT
      }
}


void delay(void)
{
    int i, num;
    XEvent event;

    num = (1000.0 / (((double)ANIMATION_SPEED / 100.0) + 0.005)) - 1000.0;

    while (XtPending() != 0)
      {
	  XtNextEvent(&event);
	  XtDispatchEvent(&event);
      }
    for (i = 1; i <= num; i++)
      {
	  while (XtPending() != 0)
	    {
		XtNextEvent(&event);
		XtDispatchEvent(&event);
	    }
	  num = (100.0 / (((double)ANIMATION_SPEED / 100.0) + 0.05)) - 100.0;
      }
}


/*
 * This function updates bin[] which is used to determine which Jacobian
 * to use based on the current vocal tract configuration.  This function
 * should be called every time an AMpar[] is changed, and certainly before
 * any zxm[] are used if there is any chance that the AMpar[] have changed.
 *
 * returns 1 if region has changed since last call, 0 otherwise.
 */
int get_tessellated_bin(void)
{
    int i;
    int region_has_changed=0;
    double par;

    /*
     * The tessellated Maeda parameters are each divided into NTi bins.
     * These integer bin specifiers are used to determine the Jacobian
     * as efficiently as possible.  The state variables are not affected
     * by this tessellation process.  AMpar[] must be set correctly when
     * this function is called.
     */

    /* determine current tessellated bin */
    for (i=0; i<7; i++) {
	par=AMpar[i];

	/* ensure that max maeda parameter is within a bin */
	if (par >= max_artic[i]) par = max_artic[i] - 0.0001;

	bin[i] = (int)((par-min_artic[i])/((max_artic[i]-min_artic[i])/NT[i]));
	if (bin[i] != old_bin[i])
		region_has_changed = 1;
    }

    if (region_has_changed) {
	for (i=0; i<7; i++)
	   old_bin[i] = bin[i];
    }

    return(region_has_changed);
}


/*
 * State variables represent feedback from the vocal tract and come
 * in opponent pairs whose sum is 1.
 *
 * Before calling this function, get_formants()
 *   must have been called.
 */

void get_state(void)
{
   normalize_formants(s,syn_targets);
}


/*** Warning:  The first array passed to normalize_formants 
  must of dimension (NUM_STATE_VARS + 1) even though the some array
  members may be filled with garbage.  Consult Michelle for
  details.  ***/
void normalize_formants(double *s,double *formants)
{
   double ratios[NUM_TARGETS],SF0;
   int i;

   SF0 = get_SF0(F0);

   if(NUM_STATE_VARS > 6){  /*magnitude info stored in state variables*/
     s[7] = formants[3]/600.0;
     s[8] = 1.0 - s[7];
   }    

   if (USE_RATIO_COORDS == 1)
     {
	if (formants[0] > f1_min)
	  {
	     ratios[0] = log10(formants[0]/SF0);
	     ratios[1] = log10(formants[1]/formants[0]);
	  }
	else
	  {
	     ratios[0] = log10(f1_min/SF0);
	     ratios[1] = log10(formants[1]/f1_min);
	  }
	ratios[2] = log10(formants[2]/formants[1]);
        if(NUM_STATE_VARS > 6) ratios[3] = formants[3]/600.0;
	if(!normalize_ratios(s,ratios)) 
	  {
		   printf("F1 = %lf, F2 = %lf F3 = %lf mag = %lf\n",
			  formants[0],formants[1],formants[2],formants[3]);
                   for(i=0;i<7;i++) printf("%lf ",AMpar[i]);printf("\n");
                   getchar();
	 }

     }
   else
     {
	 s[1] = (formants[0] - f1_min)/(f1_max - f1_min);
	 s[2] = 1.0 - s[1];
	 if (NUM_STATE_VARS > 2)
	   {
	      s[3] = (formants[1] - f2_min)/(f2_max - f2_min);
	      s[4] = 1.0 - s[3];
	   }
	 
	 if (NUM_STATE_VARS > 4)
	   {
	      s[5] = (formants[2] - f3_min)/(f3_max - f3_min);
	      s[6] = 1.0 - s[5];
	   }

	 for (i = 1; ((i <= NUM_STATE_VARS)&&(i<=6)); i++)
	   {
	      if (s[i] > 1.001) 
		{
		   s[i] = 1.0;
		   printf("State variable %d > 1.0.\n",i);
		   printf("F1 = %lf, F2 = %lf F3 = %lf mag voice = %lf\n",
			  formants[0],formants[1],formants[2],formants[3]);
		}
	      if (s[i] < 0.0) 
		{
		   s[i] = 0.0;
		   printf("State variable %d < 0.0.\n",i);
		   printf("F1 = %lf, F2 = %lf F3 = %lf\n",
			  formants[0],formants[1],formants[2]);
		}
	   }
      }
}


/*** Warning:  The first array passed to normalize_ratios
  must of dimension (NUM_STATE_VARS + 1) even though the some array
  members may be filled with garbage.  Consult Michelle for
  details.  ***/
int normalize_ratios(double *s,double *ratios)
{
    int i,j,return_val=1;
    double formants[NUM_TARGETS];
    double freq;
    static int log_initialized=0;
    static double s1_max;
    static double s1_min;
    static double s2_max;
    static double s2_min;
    static double s3_max;
    static double s3_min;
    
    double F1,F2,F3;
    double f0_max = 220, f0_min = 80;
    /*
    printf("in norm ratios: %lf %lf %lf %lf\n",ratios[0],ratios[1],ratios[2],ratios[3]);
    */
    if(NUM_STATE_VARS > 6){  /*magnitude info stored in state variables*/
      s[7] = ratios[3];
      s[8] = 1.0 - s[7];
    }    

    if (USE_RATIO_COORDS == 1)
      {
	 if (!log_initialized) {
	    s1_min = log10(f1_min/get_SF0(f0_max));
	    s1_max = log10(f1_max/get_SF0(f0_min));
	    s2_min = 0;
	    s2_max = log10(f2_max/f1_min);
	    s3_min = 0;
	    s3_max = log10(f3_max/f2_min);
	    log_initialized = 1;
	 }
	 
	 s[1] = (ratios[0] - s1_min)/(s1_max - s1_min);
	 s[2] = 1.0 - s[1];
	 if (NUM_STATE_VARS > 2)
	   {
	      s[3] = (ratios[1] - s2_min)/(s2_max - s2_min);
	      s[4] = 1.0 - s[3];
	   }
	 
	 if (NUM_STATE_VARS > 4)
	   {
	      s[5] = (ratios[2] - s3_min)/(s3_max - s3_min);
	      s[6] = 1.0 - s[5];
	   }

	 for (i = 1; ((i <= NUM_STATE_VARS)&&(i<=6)); i++)
	   {
	      if (s[i] > 1.0001) 
		{
		   printf("State variable %d > 1.0: %lf\n",i,s[i]);
		   s[i] = 1.0;
                   printf("s[1] = %lf, s[3] = %lf s[5] = %lf s[7] = %lf\n",
                        s[1],s[3],s[5],s[7]);
		   printf("logF1/SFO = %lf, logF2/F1 = %lf logF3/F2 = %lf\n",
			  ratios[0],ratios[1],ratios[2]);

                   return_val = 0;
		}
	     if (s[i] < 0)
	       s[i] = 0;
	   }
      }
    else
      {
	formants[0] = get_SF0(F0) * exp(ratios[0]);
	formants[1] = formants[0] * exp(ratios[1]);
	formants[2] = formants[1] * exp(ratios[2]);
        if(NUM_STATE_VARS > 3) formants[3] = ratios[3]*600.0;
	normalize_formants(s,formants);
      }


    /*    printf("out of norm ratios: %lf %lf %lf %lf\n",ratios[0],ratios[1],ratios[2],ratios[3]);
     */
  return(return_val);
 }


void get_x(void)
{
    int i,j,active_count=0;
    double target[NUM_STATE_VARS + 1];

    if ((FORMANT_TARGET_FLAG == 0) && (RATIO_TARGET_FLAG == 0))
      {
	 for (i = 1; i <= NUM_STATE_VARS; i++)
	   {  active_count = 0;
	      old_x[i] = x[i];
	      x[i] = 0.0 - s[i];
	      for (j = 1; j <= NUM_PHONEMES; j++)
		{
		   if (p[j] == 1)
		     {
			active_count++;
			if (active_count == 1)
			  x[i] = x[i] + (p[j] * zpx[j][i]);
		     }
		}
	   }
	 if (active_count > 1) 
	   {
	      printf("Following phonemes active simultaneously:  ");
	      for (j = 1; j <= NUM_PHONEMES; j++)
		{
		   if (p[j] == 1) printf("%c ", p_name[j]);
		}
	      printf("\n");
	   }
      }
    
    if (FORMANT_TARGET_FLAG == 1)
      {
	 normalize_formants(target,FORMANT_TARGET);
	 for (i = 1; i <= NUM_STATE_VARS; i++)
	   {
	      old_x[i] = x[i];
	      x[i] = target[i] - s[i];
	   }
      }
    if (RATIO_TARGET_FLAG == 1)
      {
	 normalize_ratios(target,RATIO_TARGET);
	 for (i = 1; i <= NUM_STATE_VARS; i++)
	   {
	      old_x[i] = x[i];
	      x[i] = target[i] - s[i];
	   }
      }
}


void rest_config(void)
{
    int j;

    for (j = 0; j < NUM_ARTICULATORS; j++)
      if (BLOCKED[j] == 0) AMpar[j] = 0.0;

    velum_height = 0;
    velum = 0;
    voicing = 0;
    noise = 0;
    pressure = 0;

    for (j = 1; j <= NUM_METHODS; j++)
	m[j] = 0.0;

    for (j = 0; j < 7; j++)
	old_bin[j] = 1000;

    if (!babbling_flag || (babbling_flag && (babbling_graphics_on > 0.5))) {
	XClearWindow(theDisplay,theWindow);
	draw_man();
	frm_replot();
	update_maeda_sliders();
	frm_plot(syn_targets);
	if (!RBF) get_tessellated_bin();
	display_formants();
    }
}


void learn_methods(void)
{
    int i,j;
    double time;
    int my_index[2];

    bin_count[bin[0]][bin[1]][bin[2]][bin[3]][bin[4]][bin[5]][bin[6]] += 1;

    for (time = 0.0; time <= trial_length; time += time_step) {
     for (i = 1; i <= NUM_STATE_VARS; i++) {
       my_index[0] = i;
       for (j = 1; j <= NUM_METHODS; j++) {
	 if (m[j] > 0.0) {
	   my_index[1] = j;
	   zxm[bin[0]][bin[1]][bin[2]][bin[3]][bin[4]][bin[5]][bin[6]][i][j]=
	      INTEGRATOR(dzxm_dt,my_index,
	      zxm[bin[0]][bin[1]][bin[2]][bin[3]][bin[4]][bin[5]][bin[6]][i][j],
	      (time - time_step),time_step);
           if(zxm[bin[0]][bin[1]][bin[2]][bin[3]][bin[4]][bin[5]][bin[6]][i][j]<
		0.0)
	     zxm[bin[0]][bin[1]][bin[2]][bin[3]][bin[4]][bin[5]][bin[6]][i][j]=
		0.0;
	 }
       }
     }
   }
}


double dzxm_dt(int *my_index, double z, double t)
{
    int i,j;
    double dz;
    double xsum = 0.0;
    double velocity_training_ratio = 0.1;

    if (coverage) velocity_training_ratio = 10.0;

#ifdef COMMENT
    dz = velocity_learning_rate * m[*(my_index + 1)] * 
                   (((- velocity_decay_rate) * z) + 
		    (old_x[*my_index] - x[*my_index]));
#endif COMMENT

    for (i = 1; i <= NUM_STATE_VARS; i++) {
	if (fabs(x[i] - old_x[i]) > 2)
	{
		printf("Formant changed too much; x[%d] - old_x[%d] = %lf\n",
			i,i,(x[i] - old_x[i]));
		printf("Maeda parameters = %lf, %lf, %lf, %lf, %lf, %lf, %lf\n",
			AMpar[0],AMpar[1],AMpar[2],AMpar[3],
			AMpar[4],AMpar[5],AMpar[6]);
		wait_for_key_stroke();
	}

	j = *(my_index + 1);
        xsum += ((x[i] - old_x[i]) *
           zxm[bin[0]][bin[1]][bin[2]][bin[3]][bin[4]][bin[5]][bin[6]][i][j]);
    }

    if(get_min_area() > 0.2){
      dz = -velocity_learning_rate * (x[*my_index] - old_x[*my_index]) *
	 (m[*(my_index + 1)] + (velocity_training_ratio * xsum));
   }
      else dz = 0;
    return (dz);
}


void learn_phonemes(void)
{
    int i,j;
    double time;
    int my_index[2];

    for (time = 0.0; time <= trial_length; time += time_step)
      {
	  for (i = 1; i <= NUM_PHONEMES; i++)
	    {
		if (p[i] == 1)
		  {
		      my_index[0] = i;

		      for (j = 1; j <= NUM_STATE_VARS; j++)
			{
			    my_index[1] = j;
			    zpx[i][j] = INTEGRATOR(dzpx_dt,my_index,zpx[i][j],
						 (time - time_step),time_step);
			}

		  }
	    }
      }
 }


double dzpx_dt(int *my_index, double z, double t)
{
    double dz,f;

#ifdef COMMENT
    /* Point attractor version:  now replaced by hypercube attractors. */
    dz = phoneme_learning_rate * p[*my_index] * 
                   (((- phoneme_decay_rate) * z) + (-(x[*(my_index + 1)])));
#endif COMMENT

    dz = (-1) * phoneme_learning_rate * p[*my_index] * 
                   (((- phoneme_decay_rate) * z) + rectify(x[*(my_index + 1)]));

    return (dz);
}


void init_weights(void)
{
    int i,j;
    int i0,i1,i2,i3,i4,i5,i6;

    for (i = 1; i <= NUM_PHONEMES; i++)
      {
	  for (j = 1; j <= NUM_STATE_VARS; j++)
	    {
		zpx[i][j] = 1.0;
	    }
      }
    if (!RBF){

    for (i0=0; i0<NT0; i0++)
     for (i1=0; i1<NT1; i1++)
      for (i2=0; i2<NT2; i2++)
       for (i3=0; i3<NT3; i3++)
        for (i4=0; i4<NT4; i4++)
         for (i5=0; i5<NT5; i5++)
          for (i6=0; i6<NT6; i6++) {

	      bin_count[i0][i1][i2][i3][i4][i5][i6] = 0;

              for (i = 1; i <= NUM_STATE_VARS; i++)
	          for (j = 1; j <= NUM_METHODS; j++)
		      zxm[i0][i1][i2][i3][i4][i5][i6][i][j] = 0.0;
	  }
 }
 }


void print_weights(FILE *outfile)
{
    int i,j;
    int i0,i1,i2,i3,i4,i5,i6;

    fprintf(outfile,"%%%%WeightFile\n"); /* have to double each % */

    for (i = 1; i <= NUM_PHONEMES; i++)
      {
	  for (j = 1; j <= NUM_STATE_VARS; j++)
	    {
		fprintf(outfile,"%lf ",zpx[i][j]);
	    }
	  fprintf(outfile,"\n");
      }
    fprintf(outfile, "\n");

#ifdef COMMENT

    for (i0=0; i0<NT0; i0++) {
     for (i1=0; i1<NT1; i1++) {
      for (i2=0; i2<NT2; i2++) {
       for (i3=0; i3<NT3; i3++) {
        for (i4=0; i4<NT4; i4++) {
         for (i5=0; i5<NT5; i5++) {
          for (i6=0; i6<NT6; i6++) {

	     fprintf(outfile,"region = %d %d %d %d %d %d %d\n",
		     i0, i1, i2, i3, i4, i5, i6);

	     fprintf(outfile,"zxm = \n");
	     for (j = 1; j <= NUM_METHODS; j++) {
               for (i = 1; i <= NUM_STATE_VARS; i++) {
	          fprintf(outfile,"%lf ",zxm[i0][i1][i2][i3][i4][i5][i6][i][j]);
		  if (zxm[i0][i1][i2][i3][i4][i5][i6][i][j] > 100.0)
			printf("very large weight\n");
	       }
	       fprintf(outfile,"\n");
             }
	     fprintf(outfile,"\n");
          }
         }
        }
       }
      }
   }
  }
#endif COMMENT

    for (i = 1; i <= NUM_PHONEMES;i++)
      {
	  fprintf(outfile,"%d ",p_voice[i]);
	  fprintf(outfile,"%d ",p_velum[i]);
	  fprintf(outfile,"%d ",p_noise[i]);
	  fprintf(outfile,"%d\n",p_pressure[i]);
      }

    fclose(outfile);
}


void read_parameter_file(FILE *infile)
{
    char param_name[40];
    double value;
    extern set_parameter();
    char temp_string[20];

    CURRENT_PARAMETER = malloc(30);

    param_count = -1;
    while (fscanf(infile,"%s", param_name) != EOF) {
	 param_count++;
	 if (!offline_babbling_flag)
		param_list[param_count] = XmStringLtoRCreate(param_name,
						      XmSTRING_DEFAULT_CHARSET);
	 fscanf(infile,"%lf",&value);
	 set_parameter(param_name,value);
    }
    if (!offline_babbling_flag) {
	sprintf(temp_string,"%0.3lf",value);
	strcpy(CURRENT_PARAMETER, param_name);
	XtVaSetValues(param_button, XmNlabelString, 
			XmStringCreateSimple(CURRENT_PARAMETER), NULL);
	XmTextSetString(param_value_string, temp_string);
    }
}


void plot_config(FILE *infile)
{
    deplot_lam();
    erase_flag = 0;

    while (fscanf(infile,"%lf %lf %lf %lf %lf %lf %lf", &AMpar[0],
		&AMpar[1], &AMpar[2], &AMpar[3], &AMpar[4], &AMpar[5],
		&AMpar[6]) != EOF) {
	find_area_function();
        plot_lam();
    }

    erase_flag = 1;
    XFlush(theDisplay);
}


void print_parameters(FILE *outfile)
{
    char *string;
    int i;
    extern double get_parameter();

    fprintf(outfile,"%%%%ParameterFile\n"); /* have to double each % */
    for (i = 0; i <= param_count; i++)
      {
        XmStringGetLtoR(param_list[i],XmSTRING_DEFAULT_CHARSET,&string);
	fprintf(outfile,"%s %lf\n", string, get_parameter(string));
    }
    fclose(outfile);
}



void load_weights(FILE *infile)
{
    int i,j;
    int i0,i1,i2,i3,i4,i5,i6;
    char infile_string[20];

    for (i = 1; i <= NUM_PHONEMES; i++)
      {
	  for (j = 1; j <= NUM_STATE_VARS; j++)
	    {
		fscanf(infile,"%lf",&zpx[i][j]);
	    }
      }
#ifdef COMMENT
     for (i0=0; i0<NT0; i0++)
      for (i1=0; i1<NT1; i1++)
       for (i2=0; i2<NT2; i2++)
        for (i3=0; i3<NT3; i3++)
         for (i4=0; i4<NT4; i4++)
          for (i5=0; i5<NT5; i5++)
           for (i6=0; i6<NT6; i6++) {
	     fscanf(infile,"%s %s",infile_string, infile_string);
             for (i=0; i<7; i++)
	         fscanf(infile,"%d ",&j);
	     fscanf(infile,"%s %s",infile_string, infile_string);
	     for (j = 1; j <= NUM_METHODS; j++)
                 for (i = 1; i <= NUM_STATE_VARS; i++)
		    fscanf(infile,"%lf",&zxm[i0][i1][i2][i3][i4][i5][i6][i][j]);
	  }
#endif COMMENT
     for (i = 1; i <= NUM_PHONEMES;i++)
      {
	  fscanf(infile,"%d",&p_voice[i]);
	  fscanf(infile,"%d",&p_velum[i]);
	  fscanf(infile,"%d",&p_noise[i]);
	  fscanf(infile,"%d",&p_pressure[i]);
      }

     fclose(infile);


 }

/********************************************************
  
    Rbf weight files must be start with the line :
    %%RbfWeightFile
    This is stripped off BEFORE calling load_rbf_weights().
    The rbf structure P must be initialized or FreeRbf() will barf 

*/


void load_rbf_weights(FILE *infile)
{
   
      printf("deallocating old rbfs\n");
      FreeRbf(P);
      printf("reading in new rbfs\n");
      ReadWeights(infile,P);
      which_fmap = 1.0;
   }


double rectify(double x)
{
    if (x < 0.0)
      return (0.0);
    else
      return (x);
}


double rk4(double ddtfunct(),int *subscripts,double xk,double tk,double dt)

/* This routine uses a 4th order Runge-Kutta method to determine the next */
/* value of xk given the name of the dxdt function, a pointer to the re-  */
/* quired subscript(s) for that function, the current xk, the current time*/
/* tk, and the time increment dt.                                         */

{
   double nextx,f1,f2,f3,f4;

   f1 = dt * ddtfunct(subscripts,xk,tk);
   f2 = dt * ddtfunct(subscripts,(xk + f1/2),(tk + dt/2));
   f3 = dt * ddtfunct(subscripts,(xk + f2/2),(tk + dt/2));
   f4 = dt * ddtfunct(subscripts,(xk + f3),(tk + dt));
   nextx = xk + ((1.0/6.0) * (f1 + (2.0 * f2) + (2.0 * f3) + f4));
   return nextx;
}



double euler(double ddtfunct(),int *subscripts,double xk,double tk,
	     double dt)

/* This routine uses Euler's method to determine the next value of xk  */
/* given the name of the dx/dt function, a pointer to the required     */
/* subscript(s) for that function, the current xk, the current time tk,*/
/* and the time increment dt.                                          */

{
   double nextx;
   nextx= xk + dt * ddtfunct(subscripts,xk,tk);
   return nextx;
}


double min(double x, double y)
{
    if (x > y)
      return y;
    else
      return x;
}

double max(double x, double y)
{
    if (y > x)
      return y;
    else
      return x;
}


int get_phoneme_index(char s)
{
    int i;

    for (i=1; i <= NUM_PHONEMES; i++)
	if (p_name[i] == s) return(i);
}

void print_zxm( void )
{
    int i,j;

    printf("zxm =\n");

    for (i = 1; i <= NUM_METHODS; i++) {
	for (j = 1; j <= NUM_STATE_VARS; j++)
	   printf("%1.4f ",
	     zxm[bin[0]][bin[1]][bin[2]][bin[3]][bin[4]][bin[5]][bin[6]][j][i]);
	printf("\n");
    }

    print_AMpar();
    print_frm();

    printf("   old_x = ");
    for (j = 1; j <= NUM_STATE_VARS; j++)
	printf("%f ", old_x[j]);
    printf("\n");

    printf("       x = ");
    for (j = 1; j <= NUM_STATE_VARS; j++)
	printf("%f ", x[j]);
    printf("\n\n");
}


int print_AMpar( void )
{
	int i;

	printf("   AMpar = ");
	for (i=0; i<7; i++)
		printf("%f ", AMpar[i]);
	printf("\n");
}


void print_frm( void )
{
	printf("      F1 = %f F2 = %f F3 = %f\n",
				syn_targets[0],syn_targets[1],syn_targets[2]);
}


void print_bin( void )
{
	printf("region = %d %d %d %d %d %d %d\n",
		bin[0],bin[1],bin[2],bin[3],bin[4],bin[5],bin[6]);
}


void region_velocity_babbling( void )
{
  int i, j, k, l, sc, q, r, mc;
  int i0, i1, i2, i3, i4, i5, i6;
  int sample_count;
  double dl[7]; /* delta length of region along i_th dimension */
  double save_maeda[7];
  double old_rate;
  int search_count;

  old_rate=rate;
  sample_count = 10;
  for (i0=0; (i0<NT0 && (BREAK != 1)); i0++)
  for (i1=0; (i1<NT1 && (BREAK != 1)); i1++)
  for (i2=0; (i2<NT2 && (BREAK != 1)); i2++)
  for (i3=0; (i3<NT3 && (BREAK != 1)); i3++)
  for (i4=0; (i4<NT4 && (BREAK != 1)); i4++)
  for (i5=0; (i5<NT5 && (BREAK != 1)); i5++)
  for (i6=0; (i6<NT6 && (BREAK != 1)); i6++) {
  printf("babbling in region %d %d %d %d %d %d %d\n",i0,i1,i2,i3,i4,i5,i6);
  rate = old_rate;

      /* set all velocities to zero */
      for (i = 1; i <= NUM_METHODS; i++)
	    m[i] = 0;

      for (i = 0; i < 7; i++)
	  dl[i] = (max_artic[i]-min_artic[i])/NT[i]/2;

      /* Put maeda articulators at center of given bin */
      AMpar[0] = min_artic[0] + (2*i0 + 1.0)*dl[0];
      AMpar[1] = min_artic[1] + (2*i1 + 1.0)*dl[1];
      AMpar[2] = min_artic[2] + (2*i2 + 1.0)*dl[2];
      AMpar[3] = min_artic[3] + (2*i3 + 1.0)*dl[3];
      AMpar[4] = min_artic[4] + (2*i4 + 1.0)*dl[4];
      AMpar[5] = min_artic[5] + (2*i5 + 1.0)*dl[5];
      AMpar[6] = min_artic[6] + (2*i6 + 1.0)*dl[6];

      get_tessellated_bin();

      /*
       * gradually approach edge of region
       * looking for valid configuration
       */
      search_count=0;
      while (check_valid_configuration() == 0) {
	  printf("searching in region...\n");
          if (search_count++ >= 3) break;
	  if (look_for_break()) break;
	  for (i = 0; i < 7; i++) {
		dl[i] = dl[i]/2;
		if (AMpar[i] == 0.0) continue;
		if (AMpar[i] < 0.0)
			AMpar[i] += dl[i];
		else
			AMpar[i] -= dl[i];
	  }
#ifdef COMMENT
process_sound_frame(); 
frm_plot(syn_targets);
update_maeda_sliders();
wait_for_key_stroke();
#endif COMMENT
	  rate = rate/2;
      }

      if (search_count >= 3) break;

      for (i=0; i<7; i++)
	  save_maeda[i] = AMpar[i];

      /* sanity check: by definition, the search must not change the bin */
      if (get_tessellated_bin()) {
		printf("talker.c: error in search\n");
		exit(1);
      }

#ifdef COMMENT
process_sound_frame();
frm_plot(syn_targets);
update_maeda_sliders();
wait_for_key_stroke();
break;
#endif COMMENT

      /* set only 1 method at a time to a nonzero value */
      for (mc = 1; ((mc <= NUM_METHODS) && (BREAK != 1)); mc++) {

        for (sc = 0; ((sc < sample_count) && (BREAK != 1)); sc++) {

	   double frm_save[11][3];
	   int vc_save[11];

           if (look_for_break()) break;

	   /* reset start position for each "smooth" movement */
           for (i=0; i<7; i++)
	       AMpar[i] = save_maeda[i];
	   get_tessellated_bin(); /* resets bin[] for subsequent tests */

           m[mc]=1; /* this is reset to zero at end of this for loop */
           for (q = 1; q <= 10; q++) {

                   apply_methods();

		   /* only compute formants once; should X10 babbling speed */
		   if (sc==0) {
			/* first time through, get all formants and save them */
			check_valid_configuration();
			for (i=0; i<2; i++)
				frm_save[q][i] = syn_targets[i];
			vc_save[q] = valid_configuration;
		   }
		   else {
			/* if not first time, use previous formant values */
			find_area_function();
			for (i=0; i<2; i++)
				syn_targets[i] = frm_save[q][i];
			valid_configuration = vc_save[q];
		   }

	           if (get_tessellated_bin()) {
			printf("breaking in smooth loop: changed region\n");
			sc--;
			rate = rate/2;
			break;
		   }
	           if (look_for_break()) break;
	           if (valid_configuration)
	           {
		     /*   get_formants(); here*/
		         get_state();
		         get_x();

		         /* "waste" first iteration to set old_x[] properly */
		         if (q > 1) learn_methods();

		         if (babbling_graphics_on > 0.5) {
			         process_sound_frame();
				 frm_plot(syn_targets);
			         display_formants();
		         }
	           }
		   else {
			printf("breaking in smooth loop: bad config\n");
			sc--;
			rate = rate/2;
			break;
		   }
	   }
	   m[mc]=0;
      }
    }
    printf("region %d %d %d %d %d %d %d  count=%d\n",
                 i0,i1,i2,i3,i4,i5,i6,bin_count[i0][i1][i2][i3][i4][i5][i6]);
    print_zxm();
    }
  rate = old_rate;
}


void random_velocity_babbling( void )
{
  int i, j;
  int noprintflag=0;

#ifdef COMMENT
  /*
   * I'm worried that get_tessellated_bin() needs to be called more
   * often and in the right places as in region_velocity_babbling().
   */
  printf("random velocity babbling currently not supported\n");
  return;
#endif COMMENT
  for (i = 1; ((i <= num_velocity_trials) && (BREAK != 1)); i++)
    {

      if (((i % 10) == 0) && !noprintflag)
	  printf("Velocity Trial=%d\n", i);
      noprintflag=0;

#ifdef COMMENT
      if (jaw_phase == 0)
	jaw_phase = 1;
      else
	jaw_phase = 0;
      get_babble_methods(jaw_phase);
#endif COMMENT
#ifdef COMMENT
      for (j = 1; j <= NUM_METHODS; j++)
	if ((random() % 3) == 1) 
	  m[j] = 1;
	else
	  m[j] = 0;
#endif COMMENT

      /*
       * For now, jaw_phase is ignored and variegated babbling is not
       * supported.
       */
      get_babble_methods(jaw_phase);
      get_x();
/*
      for (j = 1; j <= 10; j++)
*/
      for (j = 1; j <= 100; j++)
	{
	  apply_methods();
	  get_tessellated_bin();
	  check_valid_configuration();
	  if (look_for_break()) break;
	  if (valid_configuration)
	  {
	    /*  get_formants(); here*/
		get_state();
		get_x();
		if (j > 1) learn_methods(); /* sets old_x[] properly */
		if (babbling_graphics_on > 0.5) {
			process_sound_frame();
			frm_plot(syn_targets);
			display_formants();
		}
	  }
/*
	  else {
		rest_config();
		i--;
		noprintflag=1;
		break;
	  }
*/
	}
    }
}


void random_phoneme_babbling( void )
{
  int i,j,k;
  int noprintflag=0;

  for (i = 1; (i <= num_phoneme_trials && BREAK != 1); i++)
    {
      rest_config();
      if ((i % 500) == 0) printf("Trial number %d\n",i);
      look_for_break();

#ifdef COMMENT
      if (jaw_phase == 0)
	jaw_phase = 1;
      else
	jaw_phase = 0;
#endif COMMENT

      get_babble_methods(jaw_phase);
/*
      if (((i % 10) == 0) && !noprintflag)
	  printf("Phoneme Trial %d\n",i);
*/
      noprintflag=0;

      if ((random() % 3) == 1)
	{
	    /* for now, assume voicing is always on */
	    voicing = 1;
	}
      else
	{
	    voicing = 1;
	}
      if (((random() % 5) == 1) && (voicing == 1))
	{
	    velum = 1;
	}
      else
	{
	    velum = 0;
	}

      for (j = 1; j <= 10; j++)
	{
	  apply_methods();
	  check_valid_configuration();
	  if (look_for_break()) break;

	  /* use area function to find small constrictions */
	  noise = 0;
	  pressure = 0;
	  for(k=0; k<nph+nbu-1; k++) {
		if (afvt[k].A < 0.0000001)
			{
				pressure = 1;
				break;
			}
		if (afvt[k].A >= 0.0000001 && afvt[k].A < 0.01)
			{
				/* don't break; look again for closure */
				noise = 1;
			}
	     }
	  if (pressure == 1) noise = 0;

	  if (valid_configuration)
	  {
		get_phonemes();
		/*      get_formants(); here*/
		get_state();
		get_x();
		/* learn_methods(); */
		learn_phonemes();
		if (babbling_graphics_on > 0.5) {
			process_sound_frame();
			frm_plot(syn_targets);
			update_maeda_sliders();
			display_formants();
		}
	  }
	  else {
		i--;
		noprintflag=1;
		break;
	  }

	  for (k = 1; k <= NUM_PHONEMES; k++)
	    {
	      if (p[k] == 1)
		{
		  p_count[k] += 1;
		  p_voice[k] = voicing;
		  p_velum[k] = velum;
		  p_noise[k] = noise;
		  p_pressure[k] = pressure;
		}
	    }
       }
    }
}


/* return 1 if break encountered, else return 0 */
int look_for_break( void )
{
	XEvent event;
        /*  Check for a break, speed change, expose event, etc.  */
	while (XtPending() != 0)
	{
		XtNextEvent(&event);
		XtDispatchEvent(&event);
	}
	if (BREAK != 0) {
		printf("Break Encountered.\n");
		return(1);
	}
	else
		return(0);
}


void wait_for_continue( void )
{
	XEvent event;
	
	printf("Hit \"Cont after phoneme\" to continue...");
	fflush(stdout);
	while (CONTINUE_AFTER_PHONEME == 0)
		while (XtPending() != 0) {
			XtNextEvent(&event);
			XtDispatchEvent(&event);
		}
	printf("\n");
	CONTINUE_AFTER_PHONEME = 0;
}


void wait_for_key_stroke( void )
{
	printf("press any key to continue: ");
	getchar();
	printf("\n");
}


double get_SF0(double f0)
{
   return(168.0*pow((f0/168.0),0.33333333));
}


void init_ideal_targets(void)

{
   double MAX_FRMT[NUM_PHONEMES+1][NUM_TARGETS];
   double MIN_FRMT[NUM_PHONEMES+1][NUM_TARGETS];
   int i;
   double SR0;
   double x[NUM_STATE_VARS + 1],y[NUM_STATE_VARS + 1];

   SR0 = get_SF0(100.0);  /* Assuming 100Hz is "ideal" F0 */

   /*E*/ 
   IDEAL_FORMANTS[1][0] = 270;  MIN_FRMT[1][0] = 220; MAX_FRMT[1][0]=320;   
   IDEAL_FORMANTS[1][1] = 2290; MIN_FRMT[1][1] = 2140;MAX_FRMT[1][1]=2440;
   IDEAL_FORMANTS[1][2] = 3010; MIN_FRMT[1][2] = 2810;MAX_FRMT[1][2]=3510;
   /*i*/ 
   IDEAL_FORMANTS[2][0] = 390;  MIN_FRMT[2][0] = 340; MAX_FRMT[2][0]=440;
   IDEAL_FORMANTS[2][1] = 1990; MIN_FRMT[2][1] = 1890;MAX_FRMT[2][1]=2090;
   IDEAL_FORMANTS[2][2] = 2550; MIN_FRMT[2][2] = 2350;MAX_FRMT[2][2]=2950;
   /*e*/ 
   IDEAL_FORMANTS[3][0] = 530;  MIN_FRMT[3][0] = 480; MAX_FRMT[3][0]=580;
   IDEAL_FORMANTS[3][1] = 1840; MIN_FRMT[3][1] = 1740;MAX_FRMT[3][1]=1940;
   IDEAL_FORMANTS[3][2] = 2480; MIN_FRMT[3][2] = 2280;MAX_FRMT[3][2]=2980;
   /*a*/ 
   IDEAL_FORMANTS[4][0] = 660;  MIN_FRMT[4][0] = 610; MAX_FRMT[4][0]=710;
   IDEAL_FORMANTS[4][1] = 1720; MIN_FRMT[4][1] = 1620;MAX_FRMT[4][1]=1820;
   IDEAL_FORMANTS[4][2] = 2410; MIN_FRMT[4][2] = 2210;MAX_FRMT[4][2]=2910;
   /*C*/ 
   IDEAL_FORMANTS[5][0] = 730;  MIN_FRMT[5][0] = 530; MAX_FRMT[5][0]=810;
   IDEAL_FORMANTS[5][1] = 1090; MIN_FRMT[5][1] = 990; MAX_FRMT[5][1]=1190;
   IDEAL_FORMANTS[5][2] = 2440; MIN_FRMT[5][2] = 2240;MAX_FRMT[5][2]=2940;
   /*c*/ 
   IDEAL_FORMANTS[6][0] = 570;  MIN_FRMT[6][0] = 360; MAX_FRMT[6][0]=680;
   IDEAL_FORMANTS[6][1] =  840; MIN_FRMT[6][1] = 740; MAX_FRMT[6][1]=940;
   IDEAL_FORMANTS[6][2] = 2410; MIN_FRMT[6][2] = 1960;MAX_FRMT[6][2]=2910;
   /*o*/ 
   IDEAL_FORMANTS[7][0] = 440;  MIN_FRMT[7][0] = 390; MAX_FRMT[7][0]=470;
   IDEAL_FORMANTS[7][1] = 1020; MIN_FRMT[7][1] = 920; MAX_FRMT[7][1]=1120;
   IDEAL_FORMANTS[7][2] = 2240; MIN_FRMT[7][2] = 2040;MAX_FRMT[7][2]=2740;
   /*U*/ 
   IDEAL_FORMANTS[8][0] = 243/*300*/;  MIN_FRMT[8][0] = 150; MAX_FRMT[8][0]=450;
   IDEAL_FORMANTS[8][1] =  1094/*870*/; MIN_FRMT[8][1] = 800; MAX_FRMT[8][1]=1070;
   IDEAL_FORMANTS[8][2] = 2335/*2240*/; MIN_FRMT[8][2] = 1840;MAX_FRMT[8][2]=2740;
   /*u*/ 
   IDEAL_FORMANTS[9][0] = 520;  MIN_FRMT[9][0] = 490; MAX_FRMT[9][0]=570;
   IDEAL_FORMANTS[9][1] = 1190; MIN_FRMT[9][1] = 1090;MAX_FRMT[9][1]=1260;
   IDEAL_FORMANTS[9][2] = 2390; MIN_FRMT[9][2] = 2190;MAX_FRMT[9][2]=2890;
   /*I*/ 
   IDEAL_FORMANTS[10][0]= 490;  MIN_FRMT[10][0] = 390; MAX_FRMT[10][0]=590;
   IDEAL_FORMANTS[10][1]= 1350; MIN_FRMT[10][1] = 1150;MAX_FRMT[10][1]=1550;
   IDEAL_FORMANTS[10][2]= 1690; MIN_FRMT[10][2] = 1490;MAX_FRMT[10][2]= 1990;
   /*F*/ 
   IDEAL_FORMANTS[11][0]= 290;  MIN_FRMT[11][0] = 134; MAX_FRMT[11][0]=484;
   IDEAL_FORMANTS[11][1]= 686;  MIN_FRMT[11][1] = 536; MAX_FRMT[11][1]=770;
   IDEAL_FORMANTS[11][2]= 2054; MIN_FRMT[11][2] = 1854;MAX_FRMT[11][2]=2454;


   for(i=1; i<=NUM_PHONEMES;i++)
     {
	IDEAL_RATIOS[i][0] = log10(IDEAL_FORMANTS[i][0]/SR0);     
	IDEAL_RATIOS[i][1] = log10(IDEAL_FORMANTS[i][1]/IDEAL_FORMANTS[i][0]);
	IDEAL_RATIOS[i][2] = log10(IDEAL_FORMANTS[i][2]/IDEAL_FORMANTS[i][1]);
     }     
   
   for(i=1;i<=NUM_PHONEMES;i++)
     {
       MAX_RATIO[i][0] = log10(MAX_FRMT[i][0]/SR0);
       MIN_RATIO[i][0] = log10(MIN_FRMT[i][0]/SR0);
       MAX_RATIO[i][1] = log10(MAX_FRMT[i][1]/IDEAL_FORMANTS[i][0]);
       MIN_RATIO[i][1] = log10(MIN_FRMT[i][1]/IDEAL_FORMANTS[i][0]);
       MAX_RATIO[i][2] = log10(MAX_FRMT[i][2]/IDEAL_FORMANTS[i][1]);
       MIN_RATIO[i][2] = log10(MIN_FRMT[i][2]/IDEAL_FORMANTS[i][1]);
/*
       normalize_ratios(x,&MIN_RATIO[i][0]);
       normalize_ratios(y,&MAX_RATIO[i][0]);

       printf("%.3lf-%.3lf, %.3lf-%.3lf, %.3lf-%.3lf, %.3lf-%.3lf\n", 
	      x[1], y[1], y[2], x[2], x[3], y[3], y[4], x[4]);
*/

       printf("%.3lf-%.3lf,%.3lf-%.3lf,%.3lf-%.3lf\n", MIN_RATIO[i][0],
       MAX_RATIO[i][0],MIN_RATIO[i][1],MAX_RATIO[i][1],MIN_RATIO[i][2],
       MAX_RATIO[i][2]);
    }
    printf("SR0  %lf\n",SR0);

}	 


/****These routines retrieve info from the global rbf structure P******/

double get_etaB()
{
  if(P!=NULL)
    return(P->etaB);
  else
    {
     printf("ERROR - accessing etaB before RBF structure P created");
     return(0);
  }
}


double get_alpha()
{ 
 int same_alphas=1,b;
 double alpha;

  
   
  if(P!=NULL)
   {
    alpha = P->B[0].alpha;
    for(b=0;b<P->numBMaps;b++)
      {
       if(P->B[b].alpha != alpha) same_alphas = 0;
     }
    if(!same_alphas)
       printf("WARNING: different alpha values\n");
    return(P->B[0].alpha);
  }
  else
    {
     printf("ERROR - accessing alpha before RBF structure P created");
     return(0);
  }
}

