/*
 *      SenSyn - Version 1.0
 *
 *      Copyright (c) 1991, 1992 by Sensimetrics Corporation
 *      All Rights Reserved.
 *
 */

/*
 *  init_syn.c - synthesizer data structures and initialization
 *
 *  coded by Eric P. Carlson
 *
 *  Modification History:
 *     5 Nov 1991	removed from spyn.c
 */

#include "reson.h"		/* resonator support */
#include "synth.h"		/* synthesizer support */

Synthesizer synth;
Resonator glottal_pulse, spectral_tilt,
  nasal_pole_cascade, formant_1_cascade, formant_2_cascade,
  formant_3_cascade, formant_4_cascade, formant_5_cascade,
  formant_6_cascade, formant_7_cascade, formant_8_cascade, formant_2_parallel,
  formant_3_parallel, formant_4_parallel, formant_5_parallel,
  formant_6_parallel, trach_pole_cascade,
  formant_1_special, formant_2_special, formant_3_special,
  formant_4_special, nasal_pole_special, trach_pole_special;
Resonator nasal_zero_cascade, trach_zero_cascade; /* anti-resonators */
Float out[NUM_OUTPUTS];


/* ---------------------------------------------------------------------- */
/* initialize synthesizer */

void init_synthesizer()
{
  synth.parallel_only_flag = spkrdef[S_CP];
  synth.num_casc_formants = spkrdef[S_NF];
  synth.num_samples = spkrdef[S_UI];
  synth.output_select = spkrdef[S_OS];

  synth.glottis_open = 0;
  synth.period_ctr = 0;
  synth.global_time = 0;
  synth.pulse = 0;
  synth.voicing_time = 0;
  synth.voicing_amp = 0;
  synth.voicing_state = 0;
  synth.glottal_state = 0;
  synth.asp_state = 0;
  synth.integrator = 0;
  synth.random = spkrdef[S_RS];
  synth.F0 = 0;
  synth.FL = 0;
  synth.OQ = 0;
  synth.SQ = 0;
  synth.DI = 0;
  synth.AV = 0;
  synth.close_shortened = 0;
  synth.close_time = 0;

  ClearResonator(&glottal_pulse);
  ClearResonator(&spectral_tilt);
  ClearResonator(&nasal_pole_cascade);
  ClearResonator(&formant_1_cascade);
  ClearResonator(&formant_2_cascade);
  ClearResonator(&formant_3_cascade);
  ClearResonator(&formant_4_cascade);
  ClearResonator(&formant_5_cascade);
  ClearResonator(&formant_6_cascade);
  ClearResonator(&formant_7_cascade);
  ClearResonator(&formant_8_cascade);
  ClearResonator(&formant_2_parallel);
  ClearResonator(&formant_3_parallel);
  ClearResonator(&formant_4_parallel);
  ClearResonator(&formant_5_parallel);
  ClearResonator(&formant_6_parallel);
  ClearResonator(&nasal_zero_cascade);
  ClearResonator(&trach_pole_cascade);
  ClearResonator(&trach_zero_cascade);
  ClearResonator(&formant_1_special);
  ClearResonator(&formant_2_special);
  ClearResonator(&formant_3_special);
  ClearResonator(&formant_4_special);
  ClearResonator(&nasal_pole_special);
  ClearResonator(&trach_pole_special);
}
